#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2003,2006 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)46   1.363   src/csm/hacsm/pm/HAMS.pm.perl, hacsm, csm_reso, reso0611a 1/5/06 14:07:56
#
#####################################################################
#                                                                   #
# Module: HAMS.pm                                                   #
#                                                                   #
#-------------------------------------------------------------------#
#                                                                   #
# Description: This module contains Perl subroutines used by HAMS   #
#                                                                   #
# Subroutines:                                                      #
#                                                                   #
#####################################################################

package HAMS;

BEGIN
{
    if ($ENV{'CSM_HAMS_DEBUG'}) { $::HAMS_DEBUG = 1; $::VERBOSE = 1; }
    $ENV{'CSM_HAMS_CONTROL'} = 1;
    $::csmpm =
      $ENV{'CSM_PM'} ? $ENV{'CSM_PM'} : '/opt/csm/pm';    #for development use
    $ENV{'POSIXLY_CORRECT'} = 1;
    $HAMS::TIMEOUT          = 10;
    $HAMS::MIDTIME          = 50;
    $HAMS::LONGTIME         = 90;
    $HAMS::SHORTINTER       = 1;
    $HAMS::INTERVAL         = 2;
    $HAMS::MIDINTER         = 5;
    $HAMS::LONGINTER        = 15;
}

use strict;
use Fcntl qw(:flock);
use File::Basename;
use File::Path;    # Provides mkpath()
use Getopt::Long;
use lib $::csmpm;
use CLOG;
use InstallKRB5Utils;
use NetworkUtils;
use ArchiveUtils;
use ServerUtils;
my @existing_print_str;

#TODO: error checking
#TODO: fix document. a : is needed after hostname in example def file

END
{

    # get rid of KRB5 tgt if gotten
    my $exit = $?;
    HAMS->printDebug("K5ticket = $::krb5_tgt\n");
    if ($::krb5_tgt == 1)
    {
        NodeUtils->runcmd("/opt/csm/csmbin/k5destroytgt", -1);
    }
    exit $exit;
}

#######################################################################
#
# loadGlobals
#
# Note: print is allowed in this routine,
#       because MessageUtils->start_logging has not been called
# 
#######################################################################
sub loadGlobals
{
    my ($class, $nocheck) = @_;    #no check for skipping over the HAMODE value
    my $exit_code = 0;

    #GLOBAL VARIABLES
    $::MSGCAT                = 'hams.cat';
    $::MSGMAPPATH            = "$::csmroot/msgmaps";
    $::MSGSET                = 'hams';
    $::DSH                   = "/opt/csm/bin/dsh";
    $::DOMAINNAME            = "CSM_HAMS";
    $::RPDOMAIN              = "CSM_HAMS";
    $::MOUNT_POINT           = "/var/opt/hams/mnt";
    $::TB_NAME               = "HAMS_SCSI";
    $::SCRIPTS_DIR           = "/opt/csm/hams/scripts";
    $::HAMS_LOG              = "/var/log/csm/hams/hams.log";
    $ENV{'CSM_HAMS_CONTROL'} = 1;

    my $output =
      NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/lsrsrc-api -i -s IBM.DmsCtrl::::RemoteShell::SetupRemoteShell::RemoteCopyCmd::HAMode::SetupKRB5",
        0
        );
    if ($::RUNCMD_RC != 0)
    {

        # wait for 6 seconds to see whether DMS can come alive.
        # even if DMSRM crashes down, --stop commands should be run,
        # so that the locked resources can be released
        NodeUtils->runcmd("/opt/csm/csmbin/waitForRsct -t 6 IBM.DMSRM", 0);
        if (!$::RUNCMD_RC)
        {

            # Each time IBM.DMSRM is recycled, HAMode will become 3,
            # no matter what value it once was.
            # So we have to reset it on Active MS.
            if (-e "/var/opt/hams/resources/Active_CSM")
            {
                NodeUtils->runcmd(
                    "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/chrsrc-api -i -s IBM.DmsCtrl::::HAMode::1"
                    );
            }
            $output =
              NodeUtils->runcmd(
                "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/lsrsrc-api -i -s IBM.DmsCtrl::::RemoteShell::SetupRemoteShell::RemoteCopyCmd::HAMode::SetupKRB5",
                -2
                );
        }
        elsif (!$::STOP)
        {
            $exit_code = 3;
        }
    }

    (
     $::REMOTE_SHELL, $::SETUP_REMOTE_SHELL, $::REMOTECOPYCMD, $::HAMODE,
     $::SETUPKRB5
      )
      = split "::", $output;
    if (!$nocheck && $::HAMODE == 0 && !($::START || $::ACCEPT_LICENSE))
    {
        if (-f "/var/opt/hams/.paused")
        {
            print
              "HAMS is paused. Run \"hams -s\" to start and configure HAMS.\n";
        }
        else
        {
            print
              "HAMS has not been configured on this machine. Run \"hams -s\" to start and configure HAMS.\n";
        }
        if ($::LIST)
        {
            exit 0;
        }
        else
        {
            exit 2;
        }
    }
    HAMS->printDebug("$::REMOTE_SHELL, $::SETUP_REMOTE_SHELL, $::HAMODE\n");

    # this line should not be moved to the beginning of this subroutine,
    # otherwise END section of scripts will behave unexpectedly,
    # for example, Shared_Disk -m will return 255 when hams is paused
    $::HAMSDEF = "/var/opt/hams/hamsdef";

    my $osname         = $::PLTFRM;
    my $distro_name    = NodeUtils->get_DistributionName;      #requires no cmds
    my $distro_version = NodeUtils->get_DistributionVersion;   #requires LSLPP
    my $hw_arch = ArchiveUtils->get_PkgArchitecture;   #requires UNAME for Linux
    my $csm_version = "1.5.0";
    %::msOSDefs =
      ServerUtils->get_OSDefs($osname, $distro_name, $distro_version, $hw_arch,
                              $csm_version);

    # cache hardware type
    $::HWARCH = $hw_arch;

    #ENVIRONMENT VARIABLES
    $ENV{'CT_MANAGEMENT_SCOPE'} = 1;
    $::krb5_tgt                 = 0;
    $::KRB5CCNAME               = "";
    $::KRB5CCNAME               = InstallKRB5Utils->getKRB5tgt;
    HAMS->printDebug("k5 ticket = $::KRB5CCNAME\n");
    if ($::KRB5CCNAME ne "")
    {
        $ENV{'KRB5CCNAME'} = $::KRB5CCNAME;
        $::krb5_tgt = 1;
    }
    delete $ENV{'CT_MANAGEMENT_SCOPE'};
    if ($exit_code != 0 && !$nocheck)
    {
        exit $exit_code;
    }

}

#######################################################################
# getUsage
#######################################################################
sub getUsage
{
    if (!@ARGV) { HAMS->pUsage(); }

    $Getopt::Long::ignorecase = 0;    #Checks case in GetOptions
    Getopt::Long::Configure("bundling")
      ;    #allows short command line options to be grouped (e.g. -av)

    if (
        !GetOptions(
                    "h"         => \$::HELP,
                    "v|V"       => \$::VERBOSE,
                    "s|start"   => \$::START,
                    "S|stop"    => \$::STOP,
                    "l"         => \$::LIST,
                    "m"         => \$::HAMS_MOVE,
                    "i"         => \$::INSTALL,
                    "d=s"       => \$::INSTALLDIR,
                    "p|pause"   => \$::PAUSE,
                    "F"         => \$::OUTPUT_FILE,
                    "c"         => \$::COPY_LOCAL,
                    "L=s"       => \$::ACCEPT_LICENSE,
                    "r|restore" => \$::HAMS_RESTORE,
        )
      )
    {
        HAMS->pUsage();
    }

    if ($::HELP) { HAMS->pUsage(); }
    if ($::HAMS_RESTORE && !$::STOP) { HAMS->pUsage(); }
    my $count = 0;
    if ($::START)          { $count++; }
    if ($::STOP)           { $count++; }
    if ($::HAMS_MOVE)      { $count++; }
    if ($::LIST)           { $count++; }
    if ($::COPY_LOCAL)     { $count++; }
    if ($::PAUSE)          { $count++; }
    if ($::ACCEPT_LICENSE) { $count++; }
    if ($count > 1)        { HAMS->pUsage(); }
    if ($::START && !@ARGV) { $::START = "/opt/csm/hams/hamsdef"; }
    elsif ($::START && @ARGV)
    {
        $::START = $ARGV[0];
        if (!-e "$::START")
        {
            MessageUtils->message('E1', 'EMsgFileDoesNotExist', $::START);
        }
    }
    if (($::COPY_LOCAL) && (NodeUtils->isAIX()))
    {
        MessageUtils->message('E1', 'EMsgAIXNoCFlag');
    }
    if ($::OUTPUT_FILE) { undef $::VERBOSE; }    #turn off -v if -F

    if ($::ACCEPT_LICENSE)
    {
        if (scalar @ARGV > 1) { HAMS->pUsage(); }
        @::LICENSE = ($::ACCEPT_LICENSE, $ARGV[0]);
    }
}

sub pUsage
{
    MessageUtils->message('I', 'IMsgHAMSUsage');
    if ($::HELP) { exit 0; }
    exit 1;
}

#######################################################################
# getAttributes
#
# Attribute format
#    Hostname:
#    IsPrimary=<1 for primary, 0 for backup>
#    PartitionDeviceName=<Partition for storing shared CSM data>
#    FileSystemType=<Partition file system type, e.g. ext3 or jfs>
#    VolumeGroup
#    DiskDeviceName
#    SCSILockID=<The ID of SCSI device to be used for quorum determination>
#    SCSILockLUN=<The LUN of SCSI device to be used for quorum determination>
#    SCSILockHOST=<The HOST of SCSI device to be used for quorum determination>
#    SCSILockCHAN=<The CHAN of SCSI device to be used for quorum determination>
#    NIMNetworkInterface=<AIX only. The network interface to be used by the NIM master>
#    PublicIPAddress=<Optional. This is the IP alias that will be used for IPAT of the primary s connection to the public VLAN.
#                    This value should be the same for both management servers.>
#   PublicIPNetmask=<Optional. The netmask for the IP alias specified above. This value should be the same for both management servers.>
#   PublicIPAdapter=<Optional. The ethernet adapter that the Public IP address should be aliased onto. This value can be unique for each management server>
#   ClusterVLANAdapter=<Optional. The ethernet adapter that is connected to the cluster VLAN.>
#   ManagementVLANAdapter=<Optional. The ethernet adapter that is connected to the management VLAN.>
#
# Input: Filename
# Output: reference to three hashes of ms values
#######################################################################
sub getAttributes
{
    my ($class, $filename) = @_;
    HAMS->printDebug("inside getAttributes\n");

    #TODO: attribute checking
    my (%both);
    if ((!-f $filename) && ($::HAMODE != 0))
    {
        MessageUtils->message('E1', 'EMsgDefFileNotExist', $filename);
    }
    open(DEFFILE, $filename)
      or MessageUtils->message('E2', 'EMsgPerlFailed', "open", $filename, $!);
    my $line;
    my $stanza = 0;
    while ($line = <DEFFILE>)
    {
        chomp $line;
        $line =~ s/\s+//g;
        if ($line !~ m'^#' && $line =~ m/\w/)
        {    #if its not a comment and its not blank
            my $setHostname = 0;

            #$line =~ s/\s+//g;
            if ($line =~ m/(\S+):(.*)/)
            {    #this is the hostname
                if ($2 !~ m/\w/)
                {
                    HAMS->printDebug("HOSTNAME_LINE = |$line|\n");
                    $stanza++;

                    if ($stanza > 2)
                    {
                        MessageUtils->message('E1', 'EMsgInvalidHamsdefFile');
                    }
                    my $hostname = NetworkUtils->tryHost($1);
                    $both{$stanza}{'hostname'} = $hostname;
                    HAMS->printDebug("stanza $stanza has hostname $hostname\n");
                    $setHostname = 1;
                }
            }
            if ($stanza && ($setHostname == 0))
            {
                my ($attr, $value) = split(/=/, $line, 2);
                if ($value ne "")
                {
                    $both{$stanza}{$attr} = $value;
                }
            }
        }
    }
    close DEFFILE
      or MessageUtils->message('E2', 'EMsgPerlFailed', "close", $filename, $!);

    my ($primary, $backup);
    foreach my $num (keys %both)
    {
        if ($both{$num}{'IsPrimary'} == 1)
        {
            $primary = $both{$num};
        }
        else
        {
            $backup = $both{$num};
        }
    }
    if (($primary eq "") || ($backup eq ""))
    {
        MessageUtils->message('E1', 'EMsgInvalidHamsdefFile');
    }
    if (   $$primary{'PartitionDeviceName'} !~ /^\/dev\/\S+/g
        || $$backup{'PartitionDeviceName'} !~ /^\/dev\/\S+/g)
    {
        MessageUtils->message('E1', 'EMsgInvalidSharedDisk',
                              "PartitionDeviceName");
    }
    if (
        NodeUtils->isAIX()
        && (   $$primary{'PhysicalVolume'} !~ /^\/dev\/\S+/g
            || $$backup{'PhysicalVolume'} !~ /^\/dev\/\S+/g)
      )
    {
        MessageUtils->message('E1', 'EMsgInvalidSharedDisk', "PhysicalVolume");
    }
    if (
        NodeUtils->isLinux()
        && (   $$primary{'SCSILockID'} !~ /\S+/g
            || $$primary{'SCSILockLUN'} !~ /\S+/g
            || $$primary{'SCSILockHOST'} !~ /\S+/g
            || $$primary{'SCSILockCHAN'} !~ /\S+/g
            || $$backup{'SCSILockID'} !~ /\S+/g
            || $$backup{'SCSILockLUN'} !~ /\S+/g
            || $$backup{'SCSILockHOST'} !~ /\S+/g
            || $$backup{'SCSILockCHAN'} !~ /\S+/g)
      )
    {
        MessageUtils->message('E1', 'EMsgInvalidSharedDisk',
                         "SCSILockID, SCSILockLUN, SCSILockHOST, SCSILockCHAN");
    }
    if (($$primary{'CSMInstallPartition'} eq $$primary{'PartitionDeviceName'})
        || ($$backup{'CSMInstallPartition'} eq $$backup{'PartitionDeviceName'}))
    {
        MessageUtils->message('E1', 'EMsgInvalidCsmintallPartition');
    }
    if (   ($$primary{'PublicIPAddress'} =~ m/\d/)
        || ($$backup{'PublicIPAddress'} =~ m/\d/))
    {
        $::IPAT = 1;
    }

    my @machines = ($$primary{'hostname'}, $$backup{'hostname'});
    if (!$::localhost) { $::localhost = NetworkUtils->determineMS(\@machines); }
    my ($current, $remote);
    if ($::localhost eq $$primary{'hostname'})
    {

        #on primary
        $current = $primary;
        $remote  = $backup;
    }
    else
    {

        #on backup
        $current = $backup;
        $remote  = $primary;
    }

    MessageUtils->message('LI', 'IMsgHamsdefInfo', $$primary{'hostname'},
                          $$backup{'hostname'}, $$current{'hostname'});
    return ($primary, $backup, $current, $remote);
}

#######################################################################
# setupRemoteShell
# This routine sets up the remote shell from ms1 -> ms2
#
#######################################################################
sub setupRemoteShell
{
    my ($class, $ms1, $ms2) = @_;
    my $cmd;
    if ($::localhost eq $ms1)
    {
        my $reachable = 0;
        my @machines  = ($ms2);
        my $node_file =
          ServerUtils->make_node_list_file(\@machines)
          ;    #put all dest nodes in tmp file

        (my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
        if ($cmd_basename ne "rsh" && (NetworkUtils->supportedSSHversion()))
        {      #can do SSH with options
            $cmd = "$::REMOTESHELL_EXPECT -ds $node_file";
        }
        else
        {
            $cmd = "$::REMOTESHELL_EXPECT -d $node_file";
        }
        my @output = NodeUtils->runcmd($cmd, -2);
        ServerUtils->close_delete_file($::NODE_LIST_FILE, $node_file)
          ;    #delete node file
        foreach my $line (@output)
        {
            chomp $line;
            my @parts = split ":", $line;
            if (($parts[1] =~ m/test.success/) && ($parts[0] eq $ms2))
            {    #this node is reachable
                $reachable = 1;
            }
        }
        if (!$reachable && $::SETUP_REMOTE_SHELL)
        {        #SETUP REMOTE SHELL
            NetworkUtils->checkHOMEforSSHperm();

            #determine if we can setup this remote shell
            (my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
            if ($cmd_basename ne "rsh")
            {

                #check if its a supported version of OpenSSH
                if (NetworkUtils->supportedSSHversion())
                {
                    $::SSH_SUPPORT = 1;
                    HAMS->setupLocalRemoteSSH($ms1, $ms2);

                    #TODO: report error if any FAILED
                }
                else
                {    #can't setup this remote protocol
                    MessageUtils->message('E', 'IMsgNoSetupRemoteProtocol',
                                          $::REMOTE_SHELL);
                }
            }
            else
            {        #it is rsh
                HAMS->setupLocalRemoteRSH($ms1, $ms2);

                #TODO: report error if any FAILED
            }
        }
        if ($::SETUPKRB5 >= 8)
        {            #run pop5
                     #put this machines id on the other ms
            my $ms1id = NetworkUtils->getRSCThostname;
            $ms1id = lc($ms1id);
            $ms1id = "ctsrcmdp." . $ms1id;
            NodeUtils->runcmd(
                "$::DSH -t $HAMS::MIDTIME -vn $ms2 \"/opt/csm/csmbin/popk5login -p $ms1id\"",
                0
                );
        }
    }
    elsif ($::localhost eq $ms2)
    {                #check the remote shell from the other machine to this one
        my $reachable = 0;
        (my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
        if ($cmd_basename ne "rsh" && (NetworkUtils->supportedSSHversion()))
        {            #can do SSH with options
            $cmd =
              "$::DSH -t $HAMS::LONGTIME -vn $ms1 $::REMOTESHELL_EXPECT -dsn $ms2";
        }
        else
        {
            $cmd =
              "$::DSH -t $HAMS::LONGTIME -vn $ms1 $::REMOTESHELL_EXPECT -dn $ms2";
        }
        HAMS->printDebug("RUNNING: $cmd\n");
        my @output = NodeUtils->runcmd($cmd, -2);
        foreach my $line (@output)
        {
            HAMS->printDebug($line);
            chomp $line;
            my @parts = split ":", $line;
            if (($parts[2] =~ m/test.success/) && ($parts[1] =~ $ms2))
            {    #this node is reachable
                $reachable = 1;
                HAMS->printDebug("machine $ms2 is reachable\n");
            }
        }
        if (!$reachable && $::SETUP_REMOTE_SHELL)
        {
            HAMS->printDebug("machine $ms2 is NOT reachable\n");
            (my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
            if ($cmd_basename ne "rsh")
            {
                if (NetworkUtils->supportedSSHversion())
                {
                    HAMS->setupRemoteLocalSSH($ms1, $ms2);
                }
            }
            else
            {    #it is rsh
                HAMS->setupRemoteLocalRSH($ms1, $ms2);
            }
        }
        if ($::SETUPKRB5 >= 8)
        {        #run pop5
                 #put the other machines id on this ms
            my @output =
              NodeUtils->runcmd(
                "$::DSH -t $HAMS::TIMEOUT -vn $ms1 \"/usr/sbin/rsct/bin/lsnodeid\"",
                -1
                );
            my ($nothing, $ms1id) = split ": ", $output[0];
            $ms1id = lc($ms1id);
            $ms1id = "ctsrcmdp." . $ms1id;
            NodeUtils->runcmd("/opt/csm/csmbin/popk5login -p $ms1id", 0);
        }
    }
}

#######################################################################
# stopHAMSDomain
# This routine is run with HAMS is stopped  hams -S or hams -Sr 
#######################################################################
sub stopHAMSDomain
{
    my ($class, $cvals, $rvals) = @_;

    # drop the resource groups offline
    NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/chrsrc-api -e -i -o IBM.ResourceGroup::\"Active_MS\"::::NominalState::\"1\" -o IBM.ResourceGroup::\"Inactive_MS\"::::NominalState::\"1\"",
        -1
        );
    my $i = 0;

    # Active_MS should go offline in 2 times of $HAMS::LONGTIME
    my $times = 2 * $HAMS::LONGTIME / $HAMS::INTERVAL;
    while (HAMS->waitRGOffline("Active_MS") && ($i < $times))
    {
        sleep $HAMS::INTERVAL;
        $i++;
    }

    # Inactive_MS should go offline in $HAMS::LONGTIME
    $times = $HAMS::LONGTIME / $HAMS::INTERVAL;
    while (HAMS->waitRGOffline("Inactive_MS") && ($i < $times))
    {
        sleep $HAMS::INTERVAL;
        $i++;
    }
    HAMS->printDebug("All RGs offline\n");
    HAMS->removeCHARMRes($$rvals{'hostname'});
    sleep $HAMS::MIDINTER;    # allow shared disk to unmount from
                              # taking Active_MS offline
    HAMS->mountSharedDisk($cvals, $rvals);
    if ($::HAMS_RESTORE)
    {
        my $rc = HAMS->migrateBack($cvals);
        HAMS->unMountSharedDiskonCurrent($cvals);
    }
    NodeUtils->runcmd(
        "$::DSH -t $HAMS::TIMEOUT -vn $$cvals{'hostname'},$$rvals{'hostname'} \"CT_MANAGEMENT_SCOPE=1 /usr/bin/chrsrc-api -i -s IBM.DmsCtrl::::HAMode::0\"",
        0
        );
    sleep $HAMS::INTERVAL;

    # if debuging stop and do not want to lose trace files
    if ($::HAMS_DEBUG)
    {
        mkdir("/tmp/hamsdebug");
        NodeUtils->runcmd("$::CP -rp /var/ct/CSM_HAMS/log/mc/* /tmp/hamsdebug");
    }

    #check to see if $::RPDOMAIN is the current one
    my @output =
      NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=1 /usr/bin/lsrsrc-api -D ':|:' -s IBM.PeerDomain::\"Name IN ('$::RPDOMAIN')\"::Name::OpState",
        -1
        );
    if ($::RUNCMD_RC != 0)
    {
        MessageUtils->message('E', 'EMsgNoRPD', $::RPDOMAIN);
    }
    else
    {

        # stop RPD
        NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/stoprsrc-api -s IBM.PeerDomain::Name=\\\"$::RPDOMAIN\\\"::Force::1",
            -1
            );
        HAMS->waitRPD("offline");

        # remove RPD and set HAMode to zero on both servers
        my $cmd =
          'CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/rmrsrc-api -s IBM.PeerDomain::"Name==\\\\\\"'
          . $::RPDOMAIN
          . '\\\\\\""::Force::1';
        NodeUtils->runcmd(
            "$::DSH -t $HAMS::TIMEOUT -vn $$cvals{'hostname'},$$rvals{'hostname'} \"CT_MANAGEMENT_SCOPE=1 /usr/bin/chrsrc-api -i -s IBM.DmsCtrl::::HAMode::0\; $cmd\"",
            -1
            );
    }
    if (!$::HAMS_RESTORE)
    {
        HAMS->mountSharedDisk($cvals, $rvals);
    }
    NodeUtils->runcmd(
                   "CT_MANAGEMENT_SCOPE=1 /usr/bin/refrsrc-api -c IBM.DmsCtrl");
    MessageUtils->message('I', 'IMsgHAMSCleanedUp');
}

# start the rsct peer domain
sub createRPDomain
{
    my ($class, $primary, $backup) = @_;
    my $online = 0;
    NodeUtils->runcmd(
        "$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup \"/usr/bin/chrsrc-api -i -s IBM.DmsCtrl::::HAMode::3\""
        );

    #my $output = NodeUtils->runcmd('/usr/bin/lsrsrc-api -D ':|:' -n -s IBM.PeerNode::"NodeNameList IN (\'$primary\',\'$backup\')"::\'*p0x0020\'', -1);
    my $outref =
      NodeUtils->runcmd(
        "/usr/bin/lsrsrc-api -D ':|:' -n -s IBM.PeerNode::\"NodeNameList IN (\'$primary\',\'$backup\')\"::NodeNameList",
        -1, 1
        );

    if ($::RUNCMD_RC == 0)
    {

        #there is an online peer domain containing at least 1 ms
        my $found_p = 0;
        my $found_b = 0;
        foreach my $line (@$outref)
        {
            my ($nothing, $hostname) = split(/:\|:/, $line);
            $hostname =~ s/{|}//g;
            if    ($hostname eq $primary) { $found_p = 1; }
            elsif ($hostname eq $backup)  { $found_b = 1; }
        }
        if (!($found_p && $found_b))
        {    
            #one of them is not in the peer domain
            MessageUtils->message('E1', 'EMsgUselessRPD');
        }
        else
        {    #both are in there, get domain name
            $outref =
              NodeUtils->runcmd(
                "/usr/bin/lsrsrc-api -D ':|:' -s IBM.PeerDomain::::Name::OpState",
                -2, 1
                );
            foreach my $line (@$outref)
            {
                my ($name, $state) = split(/:\|:/, $line);
                if ($state == 1)
                {
                    $::RPDOMAIN = $name;
                    $online     = 1;
                    print
                      "Peer domain is already created and online: $::RPDOMAIN\n";
                    last;
                }
            }
        }
    }
    else
    {

        #There may be an offline RPD
        my $outref =
          NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=1 /usr/bin/lsrsrc-api -D ':|:' -n -s IBM.PeerDomain::\"Name IN (\'$::RPDOMAIN')\"",
            -1, 1
            );
        if ($::RUNCMD_RC != 0)
        {    #no existing peer domain
            NodeUtils->runcmd(
                "$::DSH -t $HAMS::LONGTIME -vn $primary,$backup CT_MANAGEMENT_SCOPE=2 /usr/bin/preprpnode $primary $backup"
                );

            # make RPD
            my $cmd =
              "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/mkrsrc-api \"IBM.PeerDomain::Name::$::RPDOMAIN";
            $cmd .= "::::NodeNames::{$primary,$backup}::NodeNumbers::{}\"";
            NodeUtils->runcmd($cmd);
        }
        if (!$online)
        {    #start the peer domain
            NodeUtils->runcmd(
                "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/startrsrc-api -s IBM.PeerDomain::Name=\\\"$::RPDOMAIN\\\""
                );

            #wait until domain is online
            HAMS->waitRPD("online");
        }
    }

}

###########################################################
# setupLocalRemoteSSH
#	input: \@target_nodes
###########################################################
sub setupLocalRemoteSSH
{
    my ($class, $localhost, $machine) = @_;
    my @machines = ($machine);
    NetworkUtils->rmNodesKnownHosts(\@machines);
    NodeUtils->runcmd("$::REMOTESHELL_EXPECT -k");    #make the OpenSSH keys
    NetworkUtils->cpSSHFiles();
    my $SSHdir = "/csminstall/csm/config/.ssh";
    open(FILE, ">$SSHdir/copy.perl")
      or
      MessageUtils->message('E2', 'EMsgPerlFailed', "open", "$SSHdir/copy.perl",
                            $!);

    print FILE "#!/usr/bin/perl
my \@root = split ':', (`/bin/grep ^root /etc/passwd 2>&1`);
my \$home = \$root[5];
umask(0077);
\$dest_dir = \"\$home/.ssh/\";
if (! -d \"\$dest_dir\" ) {
    # create a local directory
    \$cmd = \"$::MKDIR -p \$dest_dir\";
    system(\"\$cmd\"); 
    chmod 0700, \$dest_dir;
}
open (FILE, \"</tmp/.ssh/authorized_keys\") or die \"Failed to open file /tmp/.ssh/authorized_keys.\";
my \@contents = <FILE>;
close FILE;
open (FILE, \">>\$home/.ssh/authorized_keys\") or die \"Failed to open file \$home/.ssh/authorized_keys.\";
print FILE \@contents;
close FILE;
open (FILE, \"</tmp/.ssh/authorized_keys2\") or die \"Failed to open file /tmp/.ssh/authorized_keys2.\";
my \@contents = <FILE>;
close FILE;
open (FILE, \">>\$home/.ssh/authorized_keys2\") or die \"Failed to open file \$home/.ssh/authorized_keys2.\";
print FILE \@contents;
close FILE;
unlink \"/tmp/.ssh/authorized_keys\";
unlink \"/tmp/.ssh/authorized_keys2\";
unlink \"/tmp/.ssh/copy.perl\";
rmdir(\"/tmp/.ssh\");";
    close FILE
      or
      MessageUtils->message('E2', 'EMsgPerlFailed', "open", "$SSHdir/copy.perl",
                            $!);
    chmod 0744, "$SSHdir/copy.perl";

    #set an ENV var if more than 10 nodes
    system("$::REMOTESHELL_EXPECT -s $machine")
      ;    #has to be a system call instead of runcmd for the prompt
    my $exit = $? >> 8;
    if ($exit != 0)
    {
        NodeUtils->runcmd("$::REMOTESHELL_EXPECT -t $::REMOTE_SHELL $machine",
                          -1);
        if ($::RUNCMD_RC != 0)
        {
            MessageUtils->message('E2', 'EMsgRemoteShellSetupFailed',
                                  $::REMOTE_SHELL, $localhost, $machine);
        }
    }
    system("stty echo");
}

###########################################################
# setupLocalRemoteRSH
#	input: \@target_nodes
###########################################################
sub setupLocalRemoteRSH
{
    my ($class, $localhost, $machine) = @_;

    if (NodeUtils->isLinux())
    {

        #RH or SuSE
        #RH = $::REMOTESHELL_EXPECT -rL $MS $machine
        system("$::REMOTESHELL_EXPECT -rL $localhost $machine")
          ;    #has to be a system call instead of runcmd for the prompt
               #TODO: SuSE = $::REMOTESHELL_EXPECT -rS $MS $machine
    }
    elsif (NodeUtils->isAIX())
    {

        #AIX = $::REMOTESHELL_EXPECT -rA $MS $machine
        system("$::REMOTESHELL_EXPECT -rA $localhost $machine")
          ;    #has to be a system call instead of runcmd for the prompt
    }
    else
    {

        #TODO: MessageUtils->message('E', 'EMsgUnsupportedOS', $n, $os );
    }
    my $exit = $? >> 8;
    if ($exit != 0)
    {
        NodeUtils->runcmd("$::REMOTESHELL_EXPECT -t $::REMOTE_SHELL $machine");
        if ($::RUNCMD_RC != 0)
        {
            MessageUtils->message('E2', 'EMsgRemoteShellSetupFailed',
                                  $::REMOTE_SHELL, $localhost, $machine);
        }
    }

    system("stty echo");
}

##########################################################
# setupRemoteLocalSSH
#	input: \@target_nodes
###########################################################
sub setupRemoteLocalSSH
{
    my ($class, $remote, $local) = @_;

    #$cmd = "$::DSH -n $remote $::REMOTESHELL_EXPECT -k";
    NodeUtils->runcmd(
              "$::DSH -t $HAMS::LONGTIME -vn $remote $::REMOTESHELL_EXPECT -k");
    my $home_dir = $ENV{'HOME'};
    if (!-d "/var/opt/hams") { mkdir("/var/opt/hams"); }
    mkdir("/var/opt/hams/ssh");
    NodeUtils->runcmd(
        "$::REMOTECOPYCMD $remote:$home_dir/.ssh/id\*.pub /var/opt/hams/ssh/.");
    NodeUtils->runcmd(
        "$::CAT /var/opt/hams/ssh/identity.pub /var/opt/hams/ssh/id_rsa.pub /var/opt/hams/ssh/id_dsa.pub >> $home_dir/.ssh/authorized_keys"
        );
    unlink(</var/opt/hams/ssh/*>);
    rmdir "/var/opt/hams/ssh";
    NodeUtils->runcmd(
         "$::DSH -t $HAMS::LONGTIME -vn $remote $::REMOTESHELL_EXPECT -n $local"
         );
}

###########################################################
# setupRemoteLocalRSH
#	input: \@target_nodes
###########################################################
sub setupRemoteLocalRSH
{
    my ($class, $remote, $local) = @_;
    if (NodeUtils->isAIX())
    {
        NetworkUtils->setupAIXRSH($remote);
    }
    else
    {
        NetworkUtils->setupLinuxRSH($remote);
    }
}

###########################################################
# createHAMSResources
###########################################################
sub createHAMSResources
{
    my ($class, $cvals, $rvals, $skip_online) = @_;

    # clear the environment first
    NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/chrsrc-api -e -i -o IBM.ResourceGroup::\"Name IN ('Active_MS', 'Inactive_MS')\"::::NominalState::\"1\"",
        -1
        );

    # remove the resource groups, service IP and equivalency in one command
    NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/rmrsrc-api -i -s IBM.ResourceGroup::\"Name IN ('Active_MS','Inactive_MS')\" -s IBM.ServiceIP::\"Name IN ('HAMS_IPAT')\" -i -s IBM.Equivalency::\"Name IN ('HAMS_NET_EQU')\"",
        -1
        );

    HAMS->rmrsrc_apps($$rvals{'hostname'});

    NodeUtils->runcmd(
        "$::DSH -t $HAMS::TIMEOUT -vn $$cvals{'hostname'},$$rvals{'hostname'} \"$::RM -f /var/opt/hams/resources/*\"",
        -1
        );

    #make IBM.Application resources
    my $app_file =
      HAMS->createApplicationFile($$cvals{'hostname'}, $$rvals{'hostname'});
    NodeUtils->runcmd(
          "CT_MANAGEMENT_SCOPE=2 /usr/bin/mkrsrc -F $app_file IBM.Application");
    unlink($app_file);

    #make tiebreaker resources
    #see if the tiebreaker already exists
    my $output =
      NodeUtils->runcmd(
        "/usr/bin/lsrsrc-api -D ':|:' -n -s IBM.TieBreaker::\"Name IN ('$::TB_NAME')\"::Name",
        -1
        );
    if ($::RUNCMD_RC == 0)
    {    #TB DOES exist, remove it
        NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/chrsrc -c IBM.PeerNode OpQuorumTieBreaker=\"Operator\"",
            -1
            );
        NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=2 /usr/bin/rmrsrc -s \"Name IN ('$::TB_NAME')\" IBM.TieBreaker",
            -1
            );
    }

    #Make TB
    if (NodeUtils->isLinux())
    {
        my $id    = $$cvals{'SCSILockID'};
        my $lun   = $$cvals{'SCSILockLUN'};
        my $host  = $$cvals{'SCSILockHOST'};
        my $chan  = $$cvals{'SCSILockCHAN'};
        my $id2   = $$rvals{'SCSILockID'};
        my $lun2  = $$rvals{'SCSILockLUN'};
        my $host2 = $$rvals{'SCSILockHOST'};
        my $chan2 = $$rvals{'SCSILockCHAN'};
        NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=2 /usr/bin/mkrsrc IBM.TieBreaker Name=$::TB_NAME Type=SCSI NodeInfo='{[\"$$rvals{'hostname'}\",\"ID=$id2 LUN=$lun2 HOST=$host2,CHAN=$chan2\"],[\"$$cvals{'hostname'}\",\"ID=$id LUN=$lun HOST=$host,CHAN=$chan\"]}' HeartbeatPeriod=30"
            );
        NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=2 /usr/bin/chrsrc -c IBM.PeerNode OpQuorumTieBreaker=\"$::TB_NAME\""
            );
    }
    elsif (NodeUtils->isAIX())
    {
        my $device  = $$cvals{'PhysicalVolume'};
        my $device2 = $$rvals{'PhysicalVolume'};
        if (defined $device && defined $device2)
        {
            NodeUtils->runcmd(
                "CT_MANAGEMENT_SCOPE=2 /usr/bin/mkrsrc IBM.TieBreaker Name=$::TB_NAME Type=DISK NodeInfo='{[\"$$rvals{'hostname'}\",\"DEVICE=$device2\"], [\"$$cvals{'hostname'}\",\"DEVICE=$device\"]}' HeartbeatPeriod=30"
                );
            NodeUtils->runcmd(
                "CT_MANAGEMENT_SCOPE=2 /usr/bin/chrsrc -c IBM.PeerNode OpQuorumTieBreaker=\"$::TB_NAME\""
                );
        }
    }

    HAMS->mkrg_mkrel($cvals, $rvals);

    #change HAMode to 2
    NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/chrsrc-api -i -s IBM.DmsCtrl::::HAMode::2"
        );

    if (!$skip_online)
    {

        #Online the resource groups
        NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/chrsrc-api -e -i -o IBM.ResourceGroup::\"Active_MS\"::::NominalState::\"0\"",
            -1
            );
        sleep $HAMS::INTERVAL;
        NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/chrsrc-api -e -i -o IBM.ResourceGroup::\"Inactive_MS\"::::NominalState::\"0\"",
            -1
            );
        NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/chrsrc-api -i -s IBM.DmsCtrl::::HAMode::1"
            );
    }
}

###########################################################
#
#  waitRPD       wait for RPD come online or go offline
#
#  Arguments:
#      $status   expected RPD status after this routine returns
#
#  Returns:
#      NONE
#
###########################################################
sub waitRPD
{
    my ($class, $status) = @_;
    my ($current_status, $target_status);

    # status numbers of RPD
    # 1 online
    # 2 offline
    # 5 pending online
    # 6 pending offline
    if ($status eq "online")
    {
        $target_status  = '1';
        $current_status = '2';
    }
    elsif ($status eq "offline")
    {
        $target_status  = '2';
        $current_status = '1';
    }
    else
    {
        return 1;
    }

    MessageUtils->message('I', 'IMsgWaitingForRPD', $::RPDOMAIN, $status);

    # RPD should turn online or offline in $HAMS::LONGTIME
    my $count = 0;
    my $times = $HAMS::LONGTIME / $HAMS::INTERVAL;
    while (($current_status ne $target_status) && ($count < $times))
    {
        sleep $HAMS::INTERVAL;

        # if no RPD is defined,
        # this command is of no output
        my $output =
          NodeUtils->runcmd(
            "/usr/bin/lsrsrc-api -s IBM.PeerDomain::\"Name IN (\'$::RPDOMAIN\')\"::OpState",
            -1
            );
        if ($output eq $target_status)
        {

            # first run csmconfig to check DMS is available
            NodeUtils->runcmd(
                "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/lsrsrc-api -n -s IBM.DmsCtrl::::\'*p0x0020\'",
                -1
                );
            if (!$::RUNCMD_RC)
            {
                $current_status = $target_status;
            }
        }
        $count++;

    }

    if ($current_status eq $target_status)

      # RPD succeeds to come online or go offline
    {
        if ($status eq "offline")

          # more time has to be given when stopping RPD
        {
            sleep $HAMS::MIDINTER;
        }

        MessageUtils->message('I', 'IMsgRPDStatus', $::RPDOMAIN, $status);
    }
    else

      # the recycle time out
    {
        MessageUtils->message('E1', 'EMsgFailedTurnRPD', $::RPDOMAIN, $status);
    }

    return 0;
}

###########################################################
# locateDefFile
###########################################################
sub locateDefFile
{
    my ($class, $filename, $remote) = @_;
    if (!-d "/var/opt/hams") { mkdir("/var/opt/hams"); }
    if ($filename ne $::HAMSDEF)
    {
        NodeUtils->runcmd("$::CP $filename $::HAMSDEF");
    }
    my $dcp_cmd .= "$::REMOTECOPYCMD $filename $remote\:$::HAMSDEF";
    NodeUtils->runcmd($dcp_cmd);
}

###########################################################
# onlineWaitSrc
###########################################################
sub onlineWaitSrc
{
    my ($class, $resource, $wait_time) = @_;
    my $online = 0;
    if (!defined $wait_time) { $wait_time = $HAMS::LONGINTER; }
    my $count = 0;

    #check if online already
    my @out = NodeUtils->runcmd("LANG=C /usr/bin/lssrc -s $resource", -1);
    if (!$::RUNCMD_RC)
    {
        my ($subsys, $group, $pid, $status) = split(' ', $out[1]);
        if (defined($status) && $status eq 'active')
        {
            $online = 1;
        }
    }
    if (!$online)
    {    #wait for it to com online
        NodeUtils->runcmd("/usr/bin/startsrc -s $resource");
        while (($online == 0) && ($count < $wait_time))
        {
            if ($count < 5)
            {
                sleep $HAMS::INTERVAL;
            }
            else
            {
                sleep $HAMS::SHORTINTER;
            }
            if ($count == 0)
            {
                MessageUtils->message('I', 'IMsgWaitingSRC', $resource);
            }
            my @out = NodeUtils->runcmd("LANG=C /usr/bin/lssrc -s $resource", 0);
            my $rc  = $::RUNCMD_RC;
            if (!$rc)
            {
                my ($subsys, $group, $pid, $status) = split(' ', $out[1]);
                if (defined($status) && $status eq 'active')
                {
                    $online = 1;
                    print "\n";
                    MessageUtils->message('I', 'IMsgSRCActive', $resource);
                }
            }
            if (!$online) { print "."; }
            $count++;
        }
    }
    if (!$online) { return 1; }
    return 0;
}

###########################################################
# createApplicationFile
#    returns filename
#
# X Active_CSM_Dep IBM.Application Runs user scripts: active_csm_start.pre and
#               active_csm_stop.post
#
# X Active_CSM IBM.Application Starts, stops and monitors IBM.DMSRM
#
# X Active_CSM_Req IBM.Application Runs user scripts: active_csm_start.post and
#               active_csm_stop.pre
#
# X Shared_Disk_Dep IBM.Application Runs user scripts: shared_disk_start.pre and
#               shared_disk_stop.post
#
# X Shared_Disk IBM.Application (unless DiskRM is available)
#               Mounts, unmounts, and monitors the connection
#               to the shared disk.
#
# X Shared_Disk_Req IBM.Application Runs user scripts: shared_disk_start.post and
#               shared_disk_stop.pre
#
# X Save_CSM_Data IBM.Application Starts, stops, and monitors the ERRM conditions
#               and responses that save the CSM and RSCT data
#               to disk.
#
# X Public_IP_Dep IBM.Application Runs user scripts: public_ip_start.pre and
#               public_ip_stop.post
#
# X Public_IP_Req IBM.Application Runs user scripts: public_ip_start.post and
#               public_ip_stop.pre
#
# X Inactive_CSM IBM.Application Starts, stops and monitors IBM.DMSRM
#
# X Gather_CSM_Nodelist IBM.Application Starts, stops and monitors the condition and
#               responses that replicate the primarys nodelist and
#               setup CtSec security.
#
###########################################################
sub createApplicationFile
{
    my ($class, $current, $remote) = @_;

    my $filename   = HAMS->getRandFilename();
    my $nodestring = "'{\"$current\",\"$remote\"}'";

    #TODO: open and close error checking
    open(APPFILE, ">$filename")
      or MessageUtils->message('E2', 'EMsgPerlFailed', "open", $filename, $!);

    print APPFILE <<"END_OF_FILE";
PersistentResourceAttributes::
	resource 1:
		Name="Active_CSM"
		StartCommand="/opt/csm/csmbin/hams/Active_CSM --start" 
		StartCommandTimeout=1000
		StopCommand="/opt/csm/csmbin/hams/Active_CSM --stop"
		StopCommandTimeout=120
		MonitorCommand="/opt/csm/csmbin/hams/Active_CSM --monitor"
		MonitorCommandPeriod=20
		MonitorCommandTimeout=20
		UserName="root"
		ProtectionMode=1
		NodeNameList=$nodestring
	resource 2:
		Name="Inactive_CSM"
		StartCommand="/opt/csm/csmbin/hams/Inactive_CSM --start"
		StartCommandTimeout=300
		StopCommand="/opt/csm/csmbin/hams/Inactive_CSM --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Inactive_CSM --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 3:
		Name="Active_CSM_Dep"
		StartCommand="/opt/csm/csmbin/hams/Active_CSM_Dep --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Active_CSM_Dep --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Active_CSM_Dep --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 4:
		Name="Active_CSM_Req"
		StartCommand="/opt/csm/csmbin/hams/Active_CSM_Req --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Active_CSM_Req --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Active_CSM_Req --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 5:
		Name="Shared_Disk_Dep"
		StartCommand="/opt/csm/csmbin/hams/Shared_Disk_Dep --start"
		StartCommandTimeout=120
		StopCommand="/opt/csm/csmbin/hams/Shared_Disk_Dep --stop"
		StopCommandTimeout=120
		MonitorCommand="/opt/csm/csmbin/hams/Shared_Disk_Dep --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 6:
		Name="Shared_Disk"
		StartCommand="/opt/csm/csmbin/hams/Shared_Disk --start"
		StartCommandTimeout=360
		StopCommand="/opt/csm/csmbin/hams/Shared_Disk --stop"
		StopCommandTimeout=360
		MonitorCommand="/opt/csm/csmbin/hams/Shared_Disk --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=60
		UserName="root"
		NodeNameList=$nodestring
	resource 7:
		Name="Shared_Disk_Req"
		StartCommand="/opt/csm/csmbin/hams/Shared_Disk_Req --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Shared_Disk_Req --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Shared_Disk_Req --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 8:
		Name="Save_CSM_Data"
		StartCommand="/opt/csm/csmbin/hams/Save_CSM_Data --start"
		StartCommandTimeout=300
		StopCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Save_CSM_Data --stop"
		MonitorCommand="/opt/csm/csmbin/hams/Save_CSM_Data --monitor"
		MonitorCommandPeriod=20
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 9:
		Name="Public_IP_Dep"
		StartCommand="/opt/csm/csmbin/hams/Public_IP_Dep --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Public_IP_Dep --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Public_IP_Dep --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 10:
		Name="Public_IP_Req"
		StartCommand="/opt/csm/csmbin/hams/Public_IP_Req --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Public_IP_Req --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Public_IP_Req --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 11:
		Name="Gather_CSM_Nodelist"
		StartCommand="/opt/csm/csmbin/hams/Gather_CSM_Nodelist --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Gather_CSM_Nodelist --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Gather_CSM_Nodelist --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 12:
		Name="Active_VLAN_Connection"
		StartCommand="/opt/csm/csmbin/hams/Active_VLAN_Connection --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Active_VLAN_Connection --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Active_VLAN_Connection --monitor"
		MonitorCommandPeriod=20
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 13:
		Name="Inactive_VLAN_Connection"
		StartCommand="/opt/csm/csmbin/hams/Inactive_VLAN_Connection --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Inactive_VLAN_Connection --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Inactive_VLAN_Connection --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
END_OF_FILE
    close APPFILE
      or MessageUtils->message('E2', 'EMsgPerlFailed', "close", $filename, $!);

    return $filename;

}

##########################################################
# getRandFilename
#    returns filename
###########################################################
sub getRandFilename
{
    my $file = "/tmp/hams_$$";
    while (-e $file)
    {
        $file = ServerUtils->CreateRandomName($file);
    }
    return $file;
}

###########################################################
# migrateCSMFiles
#Initial migration of CSM data to shared disk on active management server HA CSM will create
#the following subdirectories:
#   /var/opt/hams/mnt/csminstall
#   /var/opt/hams/mnt/etc
#   /var/opt/hams/mnt/var
#Once the above directories have been created HA CSM will recursively copy the existing data from
#/csminstall, /etc/opt/csm, and /var/opt/csm onto the shared disk. Once all of the data has been
#copied, all files will be removed from the aforementioned directories and they will be used as
#symbolic links. On the active management server, the following symbolic links will be made.
#These symbolic links will allow the CSM code to access its files in the regular locations,
#while making sure they are stored on the shared disk.
#/csminstall -> /var/opt/hams/mnt/csminstall
#/etc/opt/csm -> /var/opt/hams/mnt/etc
#/var/opt/csm -> /var/opt/hams/mnt/var
###########################################################
sub migrateCSMFiles
{
    my ($class, $cvals, $rvals) = @_;

    HAMS->mountSharedDisk($cvals, $rvals, 1);    #1 = don't mountLinux local

    if (NodeUtils->isAIX())
    {                                            #use sym links for AIX
		#
        # create *local dir and  copy files to *local directories
		#
		HAMS->copyLocalDirs($cvals);

        #make sub dirs  on the shared disk
		#
        my @dirs = ("csminstall", "etc", "var", "usr");
        foreach my $d (@dirs)
        {
            if (!-d "$::MOUNT_POINT/$d") { mkdir("$::MOUNT_POINT/$d"); }
        }
        # skip making the links to the shared disk,
        # when HA MS is started the first time
        HAMS->moveDir("/csminstall",  "$::MOUNT_POINT/csminstall", 1, $cvals, 1);
        HAMS->moveDir("/etc/opt/csm", "$::MOUNT_POINT/etc",        1, $cvals, 1);
        HAMS->moveDir("/var/opt/csm", "$::MOUNT_POINT/var",        1, $cvals, 1);
    }
    else  # if Linux
    {    #use remounted directories for Linux
            #make sub dirs
        my @dirs = ("csminstall", "etc", "var", "usr");
        foreach my $d (@dirs)
        {
            if (!-d "$::MOUNT_POINT/$d") { mkdir("$::MOUNT_POINT/$d"); }
        }
        HAMS->moveMountDir("/csminstall", "$::MOUNT_POINT/csminstall", 1,
                           $cvals);
        HAMS->moveMountDir("/etc/opt/csm", "$::MOUNT_POINT/etc", 1, $cvals);
        HAMS->moveMountDir("/var/opt/csm", "$::MOUNT_POINT/var", 1, $cvals);

    }

    NodeUtils->runcmd("$::RM -rf $::MOUNT_POINT/data/\*");

    HAMS->unMountSharedDiskonCurrent($cvals);

}

#----------------------------------------------------------------------------

=head3    migrateBack 
  this routine migrate all the data from the shared disk back to local file
  systems on the hams -Svr command
=cut

#----------------------------------------------------------------------------
sub migrateBack
{
    my ($class, $cvals) = @_;
    require Cwd;
    my $orig_dir = Cwd::getcwd();
    my @dirs     = ("/csminstall", "/etc/opt/csm", "/var/opt/csm");
    my $string   = join ', ', @dirs;
    my $copied   = 0;
    MessageUtils->message('I', 'IMsgCopyingFilesBack', $string);
    foreach my $dir (@dirs)
    {
        my $skip = 0;
        if (!-l "$dir" && -d "$dir")
        {
            my @partition = HAMS->isMountedonCurrent($cvals, $dir);
            if (NodeUtils->isLinux() && ($partition[1] =~ "/var/opt/hams/mnt"))
            {
                NodeUtils->runcmd("$::UMOUNT $dir");

                my $dirOnSD;
                if ($dir eq "/csminstall")
                {
                    $dirOnSD = "$::MOUNT_POINT/csminstall";
                }elsif ($dir eq "/etc/opt/csm")
                {
                    $dirOnSD = "$::MOUNT_POINT/etc";
                }else
                {
                    $dirOnSD = "$::MOUNT_POINT/var";
                }

                opendir DIR, $dirOnSD
                    or
                    MessageUtils->message('E2', 'EMsgPerlFailed', "opendir",
                        $dirOnSD, $!);
                my @contents = readdir(DIR);
                closedir DIR
                    or
                    MessageUtils->message('E2', 'EMsgPerlFailed', "closedir",
                        $dirOnSD, $!);

                # only if we have sth to copy back, we can remove local files.
                if (scalar @contents > 2)
                {
                    NodeUtils->runcmd("$::RM -rf $dir");
                }
            }
            else
            {
                MessageUtils->message('I', 'IMsgDirExists', $dir);
                chomp(my $line = <STDIN>);
                if ($line =~ /n|N/)
                {
                    MessageUtils->message('I', 'IMsgNotRestoreDir', $dir);
                    $skip = 1;
                }
                elsif ($line =~ /y|Y/)
                {
                    MessageUtils->message('I', 'IMsgRemoveDir', $dir);
                    if (defined $$cvals{'CSMInstallPartition'})
                    {
                        NodeUtils->runcmd("$::RM -rf $dir", -1);
                        NodeUtils->runcmd("$::UMOUNT $dir", -1);
                    }
                    else
                    {
                        NodeUtils->runcmd("$::RM -rf $dir");
                    }
                }
            }
        }
        if ((!$skip) || ((-l "$dir") || (!-e "$dir")))
        {
            if (-l "$dir")
            {
                if (unlink($dir) != 1)
                {
                    MessageUtils->message('E2', 'EMsgUnlinkFailed', $dir, $!);
                }
            }
            if ($dir =~ m/csminstall/)
            {
                if (defined $$cvals{'CSMInstallPartition'})
                {
                    if (!-e $dir)
                    {
                        mkdir($dir)
                          or
                          MessageUtils->message('E2', 'EMsgPerlFailed', "mkdir",
                                                $dir, $!);
                    }
                    NodeUtils->runcmd(
                                "$::MOUNT $$cvals{'CSMInstallPartition'} $dir");
                }
                if (!-d "$::MOUNT_POINT/csminstall")
                {    #just make the directory
                    mkdir($dir)
                      or MessageUtils->message('E2', 'EMsgPerlFailed', "mkdir",
                                               $dir, $!);
                }
                else
                {
                    opendir DIR, "$::MOUNT_POINT/csminstall"
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "opendir",
                                            "$::MOUNT_POINT/csminstall", $!);
                    my @contents = readdir(DIR);
                    closedir DIR
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "closedir",
                                            "$::MOUNT_POINT/csminstall", $!);
                    if (scalar @contents > 2)
                    {
                        # "." and ".." will always be counted

                        if ($$cvals{'CSMInstallPartition'})
                        {
                            NodeUtils->runcmd("$::RM -rf /csminstall/*", -1);
                        }
                        else
                        {
                            NodeUtils->runcmd("$::RM -rf /csminstall", -1);
                        }

                        if (NodeUtils->isAIX())
                        {
                            NodeUtils->runcmd(
                                     "$::CP -Rfph $::MOUNT_POINT/csminstall /");
                        }
                        else    # linux
                        {
                            if (!-e $dir)
                            {
                                mkdir($dir)
                                  or
                                  MessageUtils->message('E2', 'EMsgPerlFailed',
                                                        "mkdir", $dir, $!);
                            }
                            if (NodeUtils->isSELinux())
                            {
                                MessageUtils->message('I', 'IMsgRunningSELinux');
                                NodeUtils->runcmd(
                                    "$::CP -Rfp  --preserve=context $::MOUNT_POINT/csminstall /"
                                    );
                            }
                            else
                            {
                                chdir($dir)
                                  or MessageUtils->message('E2', 'EMsgPerlFailed', "chdir", $dir, $!);
                                NodeUtils->runcmd(
                                    "$::TAR cf - -C $::MOUNT_POINT/csminstall . | $::TAR xvf - "
                                    );
                            }

                            #check to make sure TAR worked
                        }
                        NodeUtils->runcmd(
                                         "$::RM -rf $::MOUNT_POINT/csminstall");
                        $copied++;
                    }
                    elsif (!-e $dir)
                    {    #just make the directory
                        mkdir($dir)
                          or
                          MessageUtils->message('E2', 'EMsgPerlFailed', "mkdir",
                                                $dir, $!);
                    }
                }
            }
            elsif ($dir =~ m/etc/)
            {
                if (!-d "$::MOUNT_POINT/etc")
                {        #just make the directory
                    mkdir($dir)
                      or MessageUtils->message('E2', 'EMsgPerlFailed', "mkdir",
                                               $dir, $!);
                }
                else
                {
                    opendir DIR, "$::MOUNT_POINT/etc"
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "opendir",
                                            "$::MOUNT_POINT/etc", $!);
                    my @contents = readdir(DIR);
                    closedir DIR
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "closedir",
                                            "$::MOUNT_POINT/etc", $!);
                    if (scalar @contents > 2)
                    {

                        # "." and ".." will always be counted
                        if (!-e $dir)
                        {
                            mkdir($dir)
                              or MessageUtils->message('E2', 'EMsgPerlFailed',
                                                       "mkdir", $dir, $!);
                        }
                        chdir($dir)
                          or
                          MessageUtils->message('E2', 'EMsgPerlFailed', "chdir",
                                                $dir, $!);
                        NodeUtils->runcmd(
                            "$::TAR cf - -C $::MOUNT_POINT/etc . | $::TAR xvf - "
                            );

                        NodeUtils->runcmd("$::RM -rf $::MOUNT_POINT/etc");
                        $copied++;
                    }
                    elsif (!-e $dir)
                    {    #just make the directory
                        mkdir($dir)
                          or
                          MessageUtils->message('E2', 'EMsgPerlFailed', "mkdir",
                                                $dir, $!);
                    }
                }
            }
            elsif ($dir =~ m/var/)
            {
                if (!-d "$::MOUNT_POINT/var")
                {        #just make the directory
                    mkdir($dir)
                      or MessageUtils->message('E2', 'EMsgPerlFailed', "mkdir",
                                               $dir, $!);
                }
                else
                {
                    opendir DIR, "$::MOUNT_POINT/var"
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "opendir",
                                            "$::MOUNT_POINT/var", $!);
                    my @contents = readdir(DIR);
                    closedir DIR
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "closedir",
                                            "$::MOUNT_POINT/var", $!);
                    if (scalar @contents > 2)
                    {

                        # "." and ".." will always be counted
                        if (!-e $dir)
                        {
                            mkdir($dir)
                              or MessageUtils->message('E2', 'EMsgPerlFailed',
                                                       "mkdir", $dir, $!);
                        }
                        chdir($dir)
                          or
                          MessageUtils->message('E2', 'EMsgPerlFailed', "chdir",
                                                $dir, $!);
                        NodeUtils->runcmd(
                            "$::TAR cf - -C $::MOUNT_POINT/var . | $::TAR xvf - "
                            );

                        NodeUtils->runcmd("$::RM -rf $::MOUNT_POINT/var");
                        $copied++;
                    }
                    elsif (!-e $dir)
                    {    #just make the directory
                        mkdir($dir)
                          or
                          MessageUtils->message('E2', 'EMsgPerlFailed', "mkdir",
                                                $dir, $!);
                    }
                }
            }
        }
    }
    chdir($orig_dir);

    if ($copied < 3)
    {
        return 1;    #did not copy all directories
    }
}


###########################################################
# mountSharedDisk
###########################################################
sub mountSharedDisk
{
    my ($class, $cvals, $rvals, $dontlocal) = @_;
    if (!HAMS->isMountedonCurrent($cvals, $::MOUNT_POINT))
    {
        if (NodeUtils->isLinux())
        {
            my $rc = HAMS->check_disk_info($cvals);
            if ($rc eq 1)
            {

                # disk name may be changed
                MessageUtils->message('W', 'EMsgSearchingSharedDisk');
                my $partition = HAMS->get_disk_by_info($::INFO);
                if (!defined $partition)
                {
                    MessageUtils->message('E1', 'EMsgNoValidDisk');
                }
                HAMS->store_disk_name(
                                      $partition,
                                      $$cvals{'hostname'},
                                      $$rvals{'hostname'}
                                      );
            }
        }

        #if(NodeUtils->isLinux()){
        #run the echo SCSI commands to add and remove SCSI tiebreaker device
        #echo "scsi remove-single-device 3 0 0 0" > /proc/scsi/scsi
        #echo "scsi add-single-device 3 0 0 0" > /proc/scsi/scsi
        #    my $id =$$cvals{'SCSILockID'};
        #    my $lun =$$cvals{'SCSILockLUN'};
        #    my $host =$$cvals{'SCSILockHOST'};
        #    my $chan = $$cvals{'SCSILockCHAN'};
        #    if($id ne ""){
        #	my $string = "$host $chan $id $lun";
        #	NodeUtils->runcmd("echo \"scsi remove-single-device $string\" > /proc/scsi/scsi", -1);
        #	NodeUtils->runcmd("echo \"scsi add-single-device $string\" > /proc/scsi/scsi", -1);
        #    }
        #}

        if (NodeUtils->isAIX() && defined $$cvals{'VolumeGroup'})
        {
            my $current = $$cvals{'hostname'};
            my $remote  = $$rvals{'hostname'};

            #use alarm because sometimes dsh cannot kill child proc
            $SIG{ALRM} = sub { die "timeout expired\n" };
            eval {
                alarm(15);
                NodeUtils->runcmd(
                    "$::DSH -t $HAMS::TIMEOUT -vn $current,$remote \"/usr/sbin/umount /var/opt/hams/mnt\"",
                    -1
                    );
                alarm(0);
            };
            alarm(0);

            eval {
                alarm(15);
                NodeUtils->runcmd(
                    "$::DSH -t $HAMS::TIMEOUT -vn $current \"/usr/sbin/varyoffvg $$cvals{'VolumeGroup'}\"",
                    -1
                    );
                alarm(0);
            };
            alarm(0);

            eval {
                alarm(15);
                NodeUtils->runcmd(
                    "$::DSH -t $HAMS::TIMEOUT -vn $remote \"/usr/sbin/varyoffvg $$rvals{'VolumeGroup'}\"",
                    -1
                    );
                alarm(0);
            };
            alarm(0);

            eval {
                alarm(15);
                if (HAMS->isMountedonRemote($remote))
                {
                    MessageUtils->message('E1',
                               'EMsgShouldNotMountSharedDiskonBothMS', $remote);
                }
                alarm(0);
            };
            alarm(0);

            NodeUtils->runcmd("/usr/sbin/varyonvg $$cvals{'VolumeGroup'}", -1);
            if ($::RUNCMD_RC != 0)
            {
                my @hdisks = HAMS->get_hdisk($$cvals{'VolumeGroup'});
                HAMS->break_reservation(@hdisks);
            }
            NodeUtils->runcmd("/usr/sbin/varyonvg $$cvals{'VolumeGroup'}");
        }
        sleep $HAMS::LONGINTER;    # allow varyonvg to complete
            #mount the shared disk partition to /var/opt/hams/mnt/,
        my $partition = HAMS->get_disk_name($cvals, $$cvals{'hostname'});
        HAMS->check_disk($cvals);

        if (!-e $::MOUNT_POINT)
        {
            mkdir("/var/opt/hams");
            mkdir($::MOUNT_POINT);
        }
        if (NodeUtils->isAIX())
        {
            NodeUtils->runcmd("$::MOUNT -o rw $partition $::MOUNT_POINT");
        }
        else
        {
            NodeUtils->runcmd("$::MOUNT -w $partition $::MOUNT_POINT");
            if (!defined $dontlocal)
            {
                my @dirs = (
                            "$::MOUNT_POINT/csminstall", "/csminstall",
                            "$::MOUNT_POINT/var",        "/var/opt/csm",
                            "$::MOUNT_POINT/etc",        "/etc/opt/csm"
                            );
                foreach my $dir (@dirs)
                {
                    if (!-d $dir)
                    {
                        unlink $dir;
                        mkdir $dir;
                    }
                }

                NodeUtils->runcmd(
                       "$::MOUNT --bind $::MOUNT_POINT/csminstall /csminstall");
                NodeUtils->runcmd(
                             "$::MOUNT --bind $::MOUNT_POINT/var /var/opt/csm");
                NodeUtils->runcmd(
                             "$::MOUNT --bind $::MOUNT_POINT/etc /etc/opt/csm");
            }
            unlink(<$::MOUNT_POINT/var/tmp/conserver/*>);
        }
    }
}

###########################################################

=head3
    unMountHungSharedDisk
=cut

###########################################################
sub unMountHungSharedDisk
{
    my ($class, $pvals, $bvals, $cvals) = @_;
    my $partition = HAMS->get_disk_name($cvals, $$cvals{'hostname'});

    # we have a failed disk here this may or may not work
    NodeUtils->runcmd("$::UMOUNT $::MOUNT_POINT", 0);
    if (NodeUtils->isAIX() && defined $$cvals{'VolumeGroup'})
    {
        NodeUtils->runcmd("/usr/sbin/varyoffvg $$cvals{'VolumeGroup'}", 0);
    }
    return;
}

###########################################################
# unMountSharedDiskonCurrent
###########################################################
sub unMountSharedDiskonCurrent
{
    my ($class, $cvals, $display_errors) = @_;
    if (!defined $display_errors) { $display_errors = -2; }
    my $partition = HAMS->get_disk_name($cvals, $$cvals{'hostname'});

    if (NodeUtils->isLinux())
    {

        #umount local dirs
        foreach my $dir ("/csminstall", "/var/opt/csm", "/etc/opt/csm")
        {
            if (HAMS->isMountedonCurrent($cvals, $dir))
            {
                HAMS->unexport($dir);
                NodeUtils->runcmd("$::UMOUNT $dir");
            }
        }
    }
    if (HAMS->isMountedonCurrent($cvals, $::MOUNT_POINT))
    {

        #see if anything is exported from the shared disk
        HAMS->unexport($::MOUNT_POINT);
        NodeUtils->runcmd("$::UMOUNT $::MOUNT_POINT", $display_errors);
    }
    if (NodeUtils->isAIX() && defined $$cvals{'VolumeGroup'})
    {
        NodeUtils->runcmd("/usr/sbin/varyoffvg $$cvals{'VolumeGroup'}", 0);
    }
}

#####################################################################

=head3

    isMountedonCurrent
                    check whether the shared disk or special directory
                    is mounted on local server

    arguments:
        $cvals      reference to one hash for current MS
        $dir        the directory that may be mounted
        $check_dir  flag to check /csminstall, /var/opt/csm and /etc/opt/csm
                    1  check
                    0  no check

    returns:
        1 if mounted
        0 if not mounted

=cut

######################################################################
sub isMountedonCurrent
{
    my ($class, $cvals, $dir, $check_dir) = @_;
    HAMS->printDebug("inside HAMS->isMountedonCurrent\n");
    my $partition;
    if (!defined $dir)
    {
        $dir = $::MOUNT_POINT;
        $partition = HAMS->get_disk_name($cvals, $$cvals{'hostname'});
    }
    my $ismounted = 0;

    my $cmd = "/bin/df -P $dir";
    if ($check_dir && ($dir eq $::MOUNT_POINT) && NodeUtils->isLinux())
    {
        # On Linux, we may also check the directories are mounted
        $cmd .= " /csminstall /var/opt/csm /etc/opt/csm";
    }
    my @output = NodeUtils->runcmd($cmd, -1);
                
    if (scalar @output > 1)
    {
        my $found = 0;
        foreach my $line (@output)
        {
            HAMS->printDebug("DF line =  \"$line\"\n");
            my ($file_sys, $blocks, $used, $avail, $cap, $mount_point) =
              split(' ', $line);
            if ($mount_point =~ m/$dir/)
            {
                if (   $file_sys !~ m/$::MOUNT_POINT/
                    && $mount_point =~ m/\/csminstall/)
                {

                    # if CSMInstallPartition is mounted, we ignore it
                    next;
                }
                HAMS->printDebug("line \"$line\" matches: $dir\n");
                $found = 1;
                my ($part, $rest) = split(/\s+/, $line, 2);
                if ($partition !~ m/\w/) { $partition = $part; }
                if ($check_dir && ($dir eq $::MOUNT_POINT))
                {

                    # only when the shared disk has been mounted,
                    # check action will be executed
                    my @dirs = ("/csminstall", "/etc/opt/csm", "/var/opt/csm");
                    if (NodeUtils->isAIX())
                    {

                        # on AIX, check whether 3 links point to correct position
                        foreach my $csm_dir (@dirs)
                        {
                            if (!-l $csm_dir)
                            {
                                MessageUtils->message('W', "EMsgLinkMissed",
                                                      $csm_dir);
                                $::DISK_ABNORM = 1;
                            }
                            else
                            {
                                my $where = readlink("$csm_dir");
                                if ($where !~ m/\/var\/opt\/hams\/mnt/)
                                {
                                    MessageUtils->message('W',
                                             "EMsgWrongLinkDirection", $csm_dir,
                                             $where);
                                    $::DISK_ABNORM = 1;
                                }
                            }

                        }
                    }
                    else
                    {

                        # on Linux, check whether 3 directories are mounted
                        my $content = join "", @output;
                        foreach my $csm_dir (@dirs)
                        {
                            if ($content !~ m/\Q$csm_dir\E/)
                            {
                                MessageUtils->message('W',
                                            "EMsgDirShouldBeMounted", $csm_dir);
                                $::DISK_ABNORM = 1;
                            }
                        }
                    }
                }
                last;
            }
        }
        if ($found)
        {
            $ismounted = 1;
            my @left_str = grep (/$dir/, @existing_print_str);
            if ($dir eq $::MOUNT_POINT || !scalar(@left_str))
            {
                MessageUtils->message('L', "IMsgDiskDirMounted", $partition,
                                      $dir);
                push @existing_print_str, $dir;
            }
        }
    }
    if ($dir eq $::MOUNT_POINT)
    {
        HAMS->printDebug("ismounted = $ismounted\n");
    }

    if (wantarray)
    {
        return ($ismounted, $partition);
    }
    else
    {
        return $ismounted;
    }
}

###########################################################
# printDebug
###########################################################
sub printDebug
{
    my ($class, $string) = @_;

    #print "inside debug $::HAMS_DEBUG\n";
    if ($::HAMS_DEBUG)
    {
        print "\tHAMS_DEBUG: $string";
    }
}

sub listCondResp
{
    my %conditions = (
        "IBM.ManagedNodeChanged" =>
          qq(ResourceClass::IBM.ManagedNode::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.ManagedNode Changed"::Severity::0::ManagementScope::1),
        "IBM.AssociationChanged" =>
          qq(ResourceClass::IBM.Association::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.Association Changed"::Severity::0::ManagementScope::1),
        "IBM.ConditionChanged" =>
          qq(ResourceClass::IBM.Condition::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.Condition Changed"::Severity::0::ManagementScope::1),
        "IBM.EventResponseChanged" =>
          qq(ResourceClass::IBM.EventResponse::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.EventResponse Changed"::Severity::0::ManagementScope::1),
        "IBM.SensorChanged" =>
          qq(ResourceClass::IBM.Sensor::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.Sensor Changed"::Severity::0::ManagementScope::1),
        "IBM.NodeGroupChanged" =>
          qq(ResourceClass::IBM.NodeGroup::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.NodeGroup Changed"::Severity::0::ManagementScope::1),
        "IBM.DmsCtrlChanged" =>
          qq(ResourceClass::IBM.DmsCtrl::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.DmsCtrl Changed"::Severity::0::ManagementScope::1),
        "IBM.HwCtrlPointChanged" =>
          qq(ResourceClass::IBM.HwCtrlPoint::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.HwCtrlPoint Changed"::Severity::0::ManagementScope::1),
        "IBM.DeviceGroupChanged" =>
          qq(ResourceClass::IBM.DeviceGroup::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.DeviceGroup Changed"::Severity::0::ManagementScope::1),
        "IBM.DeviceHwCtrlChanged" =>
          qq(ResourceClass::IBM.DeviceHwCtrl::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.DeviceHwCtrl Changed"::Severity::0::ManagementScope::1),
        "IBM.LPCommandsChanged" =>
          qq(ResourceClass::IBM.LPCommands::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.LPCommands Changed"::Severity::0::ManagementScope::1),
        "NIMDatabaseChanged" =>
          qq(ResourceClass::IBM.Sensor::EventExpression::"String!=String\@P"::EventDescription::"NIM Database Changed"::SelectionString::Name=\\"NIMDatabaseChanged\\"::Severity::0::ManagementScope::1),

          );

    # These are the definitions of the responses that will be created.  This is a hash in
    # which the key is the name of the response and the value is all the attribute
    # name & value pairs separated by double colons (in the mkrsrc-api format) - in this
    # case only the action array.
    my %responses = (
        HAMSSaveData =>
          qq(Actions::'{["/opt/csm/csmbin/hams/HAMSSaveData",{127},{0},{86400},"/opt/csm/csmbin/hams/HAMSSaveData",3,0,0,0,{},1]}'),
        HAMSSaveNIMDB =>
          qq(Actions::'{["/opt/csm/csmbin/hams/HAMSSaveData --nim",{127},{0},{86400},"/opt/csm/csmbin/hams/HAMSSaveData --nim",3,0,0,0,{},1]}'),
          );

    # These are the condition and response pairs that should be activated by default.
    # This is an hash, where the key consists of '"condition" "response"'
    # The double quotes around condition and response are required, as that is how
    # it is displayed by lscondresp
    my %condresp = (
                    '"IBM.ManagedNodeChanged" "HAMSSaveData"'   => 1,
                    '"IBM.AssociationChanged" "HAMSSaveData"'   => 1,
                    '"IBM.ConditionChanged" "HAMSSaveData"'     => 1,
                    '"IBM.EventResponseChanged" "HAMSSaveData"' => 1,
                    '"IBM.SensorChanged" "HAMSSaveData"'        => 1,
                    '"IBM.NodeGroupChanged" "HAMSSaveData"'     => 1,
                    '"IBM.DmsCtrlChanged" "HAMSSaveData"'       => 1,
                    '"IBM.HwCtrlPointChanged" "HAMSSaveData"'   => 1,
                    '"IBM.DeviceHwCtrlChanged" "HAMSSaveData"'  => 1,
                    '"IBM.DeviceGroupChanged" "HAMSSaveData"'   => 1,
                    '"IBM.LPCommandsChanged" "HAMSSaveData"'    => 1,
                    '"NIMDatabaseChanged" "HAMSSaveNIMDB"'      => 1,
                    );

    my %sensors =
      (NIMDatabaseChanged =>
        qq(Command::"/opt/csm/csmbin/mtime /var/adm/nim/nimdb.changed"::UserName::root::RefreshInterval::15),
        );

    if (!NodeUtils->isAIX())
    {

        #delete sensor, condition, response and association for NIM
        delete $sensors{'NIMDatabaseChanged'};
        delete $conditions{'NIMDatabaseChanged'};
        delete $responses{'HAMSSaveNIMDB'};
        delete $condresp{'"NIMDatabaseChanged" "HAMSSaveNIMDB"'};
    }

    return (\%conditions, \%responses, \%condresp, \%sensors);
}

###########################################################
# createCondResp
###########################################################
sub createCondResp
{

    my ($conditions, $responses, $condresp, $sensors) = HAMS->listCondResp();

    HAMS->queryCondResp($conditions, 'Condition');
    HAMS->queryAssociation($condresp);
    HAMS->queryCondResp($responses, 'EventResponse');
    HAMS->queryCondResp($sensors,   'Sensor');

    HAMS->defineCondResp($sensors,    'Sensor');
    HAMS->defineCondResp($conditions, 'Condition');
    HAMS->defineCondResp($responses,  'EventResponse');
    HAMS->startAssociation($condresp);

}

sub deleteCondResp
{
    my ($conditions, $responses, $condresp) = HAMS->listCondResp();

    HAMS->rmAssociation($condresp);
    HAMS->removeCondResp($conditions, 'Condition');
    HAMS->removeCondResp($responses,  'EventResponse');

}

sub queryCondResp
{
    my ($class, $hashref, $resclass) = @_;
    my $outref =
      HAMS->runCondResp(
                        'lsrsrc-api',
                        "-i -D ':|:'",
                        "-s IBM.${resclass}::::Name"
                        );    # Get the current list of resources
    foreach my $g (@$outref)
    {
        delete $$hashref{$g};
    }                         # Remove all these entries from our hash
}

sub defineCondResp
{
    my ($class, $hashref, $resclass) = @_;
    if (!scalar(%$hashref)) { return; }    # query() must have deleted them all

    # Build the mkrsrc-api argument string with all the conditions in it
    my $s = '';
    foreach my $k (keys %$hashref)
    {
        my $attrs = $$hashref{$k};
        $s .= qq( IBM.${resclass}::Name::"$k"::$attrs);
    }

    HAMS->runCondResp('mkrsrc-api', '', $s);
}

sub removeCondResp
{
    my ($class, $hashref, $resclass) = @_;

    # Use the list of keys in the above hash to delete all the predefined resources which have no associations
    my $output = NodeUtils->runcmd("/usr/sbin/rsct/bin/lscondresp", 0);
    my @redundant_resources = grep { $output !~ $_ } keys(%$hashref);
    if (!scalar(@redundant_resources))
    {
        return;
    }

    my $where = q/"Name IN ('/ . join("','", @redundant_resources) . q/')"/;
    my $cmd = "/usr/bin/chrsrc-api -s IBM.${resclass}::$where" . "::Locked::0";
    NodeUtils->runcmd($cmd, 0);
    $cmd = "/usr/bin/rmrsrc-api -i -s IBM.${resclass}::$where";
    NodeUtils->runcmd($cmd, 0);
}

sub rmAssociation
{
    my ($class, $hashref) = @_;
    foreach my $assoc (keys %$hashref)
    {
        NodeUtils->runcmd("/usr/sbin/rsct/bin/stopcondresp -U $assoc", 0);
        NodeUtils->runcmd("/usr/sbin/rsct/bin/rmcondresp $assoc",      0);
    }
}

sub startAssociation
{
    my ($class, $hashref) = @_;
    foreach my $assoc (keys %$hashref)
    {
        NodeUtils->runcmd("/usr/sbin/rsct/bin/startcondresp $assoc",    0);
        NodeUtils->runcmd("/usr/sbin/rsct/bin/startcondresp -L $assoc", 0);
    }
}

#see if any of our associations already exist
sub queryAssociation
{
    my ($class, $hashref) = @_;
    my @out = NodeUtils->runcmd("/usr/sbin/rsct/bin/lscondresp", 0);
    shift @out;    #first line is a comment
    shift @out;    #the second line is a comment
    foreach my $line (@out)
    {
        chomp $line;
        my ($condition, $response, $node, $state) = split ' ', $line;
        my $key = "$condition $response";
        delete $$hashref{$key};
    }
}

sub runCondResp
{
    my ($class, $rmccmd, $options, $resources) = @_;

    #$rmccmd = "CT_MANAGEMENT_SCOPE=1 " . $rmccmd;
    my $current_scope = $ENV{'CT_MANAGEMENT_SCOPE'};
    $ENV{'CT_MANAGEMENT_SCOPE'} = 1;
    my $outref = NodeUtils->runrmccmd($rmccmd, $options, $resources);
    my $rc = $::RUNCMD_RC;
    $::maxrc = ($rc > $::maxrc ? $rc : $::maxrc);
    $ENV{'CT_MANAGEMENT_SCOPE'} = $current_scope;
    return $outref;
}

sub determineActive
{
    my ($class, $pvals, $bvals) = @_;
    if (defined $HAMS::Active_MS) { return $HAMS::Active_MS; }    #cached
    if (!defined $pvals || !defined $bvals)
    {
        ($pvals, $bvals) = HAMS->getAttributes("/var/opt/hams/hamsdef");
    }
    my $active;
    my $cmd =
      "CT_MANAGEMENT_SCOPE=2 /usr/bin/lsrsrc-api -D ':|:' -i -s IBM.Application::\"Name IN ('Active_CSM')\"::Name::NodeNameList::OpState";
    my @output = NodeUtils->runcmd($cmd, -1);
    if ($::RUNCMD_RC != 0) { return 1; }
    foreach my $line (@output)
    {
        HAMS->printDebug("line = $line\n");
        my ($resource, $nodenamelist, $opstate) = split ':\|:', $line;
        HAMS->printDebug("opstate = $opstate\n");
        if ($opstate == 1)
        {
            if ($nodenamelist !~ /,/)
            {    #not a list
                $nodenamelist =~ s/{|}//g;
                HAMS->printDebug("nodenamelist = $nodenamelist\n");
                $active = $nodenamelist;
            }
        }
    }
    $HAMS::Active_MS = $active;    #cache it
    return $active;
}

sub determineInactive
{
    my ($class, $pvals, $bvals) = @_;
    if (defined $HAMS::Inactive_MS) { return $HAMS::Inactive_MS; }    #cached
    if (!defined $pvals || !defined $bvals)
    {
        ($pvals, $bvals) = HAMS->getAttributes("/var/opt/hams/hamsdef");
    }
    my $primary   = $$pvals{'hostname'};
    my $backup    = $$bvals{'hostname'};
    my $active_ms = HAMS->determineActive($pvals, $bvals);
    my $inactive_ms;
    if ($active_ms eq $primary) { $inactive_ms = $backup; }
    else { $inactive_ms = $primary; }
    $HAMS::Inactive_MS = $inactive_ms;                                #cache it
    return $inactive_ms;
}

sub waitAppOffline
{

    my ($class, $pvals, $bvals, $resource) = @_;
    my $active;
    my $cmd =
      "CT_MANAGEMENT_SCOPE=2 /usr/bin/lsrsrc-api -D ':|:' -i -s IBM.Application::\"Name IN (\'$resource\')\"::Name::NodeNameList::OpState";
    my @output = NodeUtils->runcmd($cmd, 0);
    if ($::RUNCMD_RC) { return 0; }
    foreach my $line (@output)
    {
        my ($resource, $nodenamelist, $opstate) = split ':\|:', $line;
        if ($opstate == 1)
        {
            if ($nodenamelist !~ /,/)
            {    #not a list
                $nodenamelist =~ s/{|}//g;
                $active = $nodenamelist;
            }
        }
    }
    if ($active)
    {
        return 1;
    }
    else
    {
        return 0;    #offline
    }
}

sub waitRGOffline
{

    my ($class, $resource) = @_;
    my $active;
    my $cmd =
      "CT_MANAGEMENT_SCOPE=2 /usr/bin/lsrsrc-api -D ':|:' -i -s IBM.ResourceGroup::\"Name IN (\'$resource\')\"::Name::OpState";
    my @output = NodeUtils->runcmd($cmd, 0);
    if ($::RUNCMD_RC) { return 0; }
    foreach my $line (@output)
    {
        my ($resource, $opstate) = split ':\|:', $line;
        if ($opstate != 2)
        {

            # 2 means offline
            $active = 1;
        }
    }
    if ($active)
    {
        return 1;
    }
    else
    {
        return 0;    #offline
    }
}
#------------------------------------------------------------------------
#  This routine creates the /var/opt/hams/*.local directores and
#  copies the local files into it
#------------------------------------------------------------------------

sub copyLocalDirs
{
  my ($class,$cvals) = @_;
  my $local_dir = "/var/opt/hams";
  my @dirs = ("csminstall.local", "etc.local", "var.local");
  foreach my $d (@dirs)  # create the directories
  {
            if (!-d "$local_dir/$d") { mkdir("$local_dir/$d"); }
  }

  # 
  #  make a local copy of /csminstall
  # 

  my $new="/var/opt/hams/csminstall.local";
  my $orig="/csminstall";
  NodeUtils->runcmd("$::RM -rf $new");
  NodeUtils->runcmd(
       "$::CP -Rfph $orig /var/opt/hams/");
  NodeUtils->runcmd(
    "$::MV /var/opt/hams/csminstall /var/opt/hams/csminstall.local");

  # 
  #  make a local copy of /var/opt/csm 
  # 

  $new="/var/opt/hams/var.local";
  $orig="/var/opt/csm";
  unlink
   "/var/opt/hams/var.local/rconsoleUpdate.rmnode_list";
  unlink
   "/var/opt/hams/var.local/rconsoleUpdate.node_list";
  unlink(</var/opt/hams/var.local/.*>);
  unlink(</var/opt/hams/var.local/tmp/*sem>);
  chdir($new);  # go to /var/opt/hams/var.local and put in files
  NodeUtils->runcmd(
    "$::TAR cf - -C $orig . | $::TAR xvf - ");

  # 
  #  make a local copy of /etc/opt/csm 
  # 

  $new="/var/opt/hams/etc.local";
  $orig="/etc/opt/csm";
  chdir($new);  # go to /var/opt/hams/etc.local and put in files
  NodeUtils->runcmd(
    "$::TAR cf - -C $orig . | $::TAR xvf - ");
}
sub createLocalDirs
{
    my ($class, $link, $cvals) = @_;
    if (NodeUtils->isLinux()) { return; }
    my $local_dir = "/var/opt/hams";
    my @dirs = ("csminstall.local", "etc.local", "var.local");
    foreach my $d (@dirs)
    {
        if (!-d "$local_dir/$d") { mkdir("$local_dir/$d"); }
    }
    HAMS->moveDir("/csminstall",  "$local_dir/csminstall.local", $link, $cvals);
    HAMS->moveDir("/etc/opt/csm", "$local_dir/etc.local",        $link, $cvals);
    HAMS->moveDir("/var/opt/csm", "$local_dir/var.local",        $link, $cvals);
}

sub moveDir
{
    my ($class, $orig, $new, $move, $cvals, $nolink) = @_;
    require Cwd;
    my $orig_dir = Cwd::getcwd();
    if (!defined $HAMS::moveDir_disp)
    {
        MessageUtils->message('I', 'IMsgCopyToSharedDisk');
        $HAMS::moveDir_disp = 1;
    }
    HAMS->printDebug(
                    "inside moveDir: orig = $orig\tnew = $new\tmove = $move\n");
    if (defined $orig && defined $new)
    {
        if (-e "$orig")
        {
            HAMS->printDebug("$orig exists!\n");
            if (-l "$orig")
            {    #then maybe its already a symbolic link to $destdir
                my $where = readlink("$orig");
                HAMS->printDebug("$orig is a symbolic link to $where\n");
                $where =~ s/\/\Z//
                  ;   #remove the last slash because it messes up the comparison
                if ($where ne $new)
                {
                    HAMS->printDebug("$where is ne to $new: unlink $orig");
                    unlink("$orig");
                    $move = 0;
                }
                else
                {

                    #this link is already correct;
                    return 0;
                }
            }
            if ($move)
            {

                #make sure there are files in there
                if (-d $orig)
                {
                    opendir DIR, $orig
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "opendir",
                                            $orig, $!);
                    my @contents = readdir(DIR);
                    closedir DIR
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "closedir",
                                            $orig, $!);
                    if (scalar @contents > 2)
                    {
                        if (   NodeUtils->isAIX()
                            && $orig =~ /csminstall/
                            && $new !~ /csminstall.local/)
                        {
                            NodeUtils->runcmd("$::RM -rf $new");
                            NodeUtils->runcmd(
                                         "$::CP -Rfph $orig /var/opt/hams/mnt");
                        }
                        elsif (NodeUtils->isAIX() && $orig =~ /csminstall/)
                        {    #csminstall.local
                            NodeUtils->runcmd("$::RM -rf $new");
                            NodeUtils->runcmd(
                                            "$::CP -Rfph $orig /var/opt/hams/");
                            NodeUtils->runcmd(
                                "$::MV /var/opt/hams/csminstall /var/opt/hams/csminstall.local"
                                );
                        }
                        else
                        {
                            mkdir($new);
                            chdir($new);
                            if ($new =~ m/var.local/)
                            {
                                unlink
                                  "/var/opt/hams/var.local/rconsoleUpdate.rmnode_list";
                                unlink
                                  "/var/opt/hams/var.local/rconsoleUpdate.node_list";
                                unlink(</var/opt/hams/var.local/.*>);
                                unlink(</var/opt/hams/var.local/tmp/*sem>);
                            }
                            NodeUtils->runcmd(
                                      "$::TAR cf - -C $orig . | $::TAR xvf - ");
                        }

                    }
                }
            }
            if (   ($orig eq "/csminstall")
                && (defined $$cvals{'CSMInstallPartition'}))
            {
                my $output = NodeUtils->runcmd("$::UMOUNT $orig", 0);
                if ($::RUNCMD_RC != 0)
                {    
                    #UMOUNT FAILED
                    if ((NodeUtils->isAIX() && $output !~ m/not find anything to unmount/) || (NodeUtils->isLinux() && $output !~ m/not mounted/))
                    {
                        # maybe the directory has not been mounted
                        MessageUtils->message('E1', 'IMsgUmountFailed');
                    }
                }
            }
            # on AIX, we can not remove current directory
            chdir("/");
            NodeUtils->runcmd("$::RM -rf $orig");
        }
        else
        {            #doesn't exist but might be a broken link
            NodeUtils->runcmd("$::RM -rf $orig");
        }
        if (!$nolink)
        {
            NodeUtils->runcmd("$::LN -s $new $orig");
        }
    }

    chdir($orig_dir);
}

#----------------------------------------------------------------------------

=head3   moveMountDir 
  This routine moves the data to the shared disk from the local file
  system on Linux 

  The three directories are NOT mounted to the shared disk
=cut
#----------------------------------------------------------------------------
sub moveMountDir
{
    my ($class, $orig, $new, $move, $cvals) = @_;
    require Cwd;
    my $orig_dir = Cwd::getcwd();

    HAMS->printDebug(
               "inside moveMountDir: orig = $orig\tnew = $new\tmove = $move\n");
    if (defined $orig && defined $new)
    {
        if (-e "$orig")
        {
            HAMS->printDebug("$orig exists!\n");

            #see if its mounted
            my @partition = HAMS->isMountedonCurrent($cvals, $orig);
            if ($partition[1] =~ "/var/opt/hams/mnt") { return 0; }
            if (-l "$orig")
            {    #then maybe its already a symbolic link to $destdir
                unlink("$orig");
            }
            elsif ($move)
            {

                #make sure there are files in there
                if (-d $orig)
                {
                    opendir DIR, $orig
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "opendir",
                                            $orig, $!);
                    my @contents = readdir(DIR);
                    closedir DIR
                      or
                      MessageUtils->message('E2', 'EMsgPerlFailed', "closedir",
                                            $orig, $!);
                    if (scalar @contents > 2)
                    {
                        if (NodeUtils->isAIX() && $orig =~ /csminstall/)
                        {

                            #this should never happen since this routine is not called on AIX
                            #however I am leaving it in just in case
                            NodeUtils->runcmd("$::RM -rf $new");
                            NodeUtils->runcmd(
                                         "$::CP -Rfph $orig /var/opt/hams/mnt");
                        }
                        else    # linux
                        {
                            mkdir($new);

                            # if secure Linux
                            if ((NodeUtils->isSELinux()) && ($orig eq "/csminstall"))
                            {
                                MessageUtils->message('I', 'IMsgRunningSELinux');
                                NodeUtils->runcmd("$::RM -rf $new");
                                NodeUtils->runcmd(
                                    "$::CP -Rfp --preserve=context $orig /var/opt/hams/mnt"
                                    );

                            }
                            else
                            {
                                chdir($new);
                                NodeUtils->runcmd(
                                      "$::TAR cf - -C $orig . | $::TAR xvf - ");
                                chdir($orig_dir);
                            }
                        }
                    }
                }
            }
            if (   ($orig eq "/csminstall")
                && (defined $$cvals{'CSMInstallPartition'}))
            {
                NodeUtils->runcmd("$::UMOUNT $orig", 0);
                if ($::RUNCMD_RC != 0)
                {    #UMOUNT FAILED
                    MessageUtils->message('E1', 'IMsgUmountFailed');
                }
            }

            #NodeUtils->runcmd("$::RM -rf $orig");
        }
        else
        {            #doesn't exist but might be a broken link
            unlink($orig);
        }
        if (!-d $orig) { mkdir($orig); }
    }
}

sub getManagementServerAttr
{
    my ($class, $node) = @_;
    if (!defined %HAMS::MSHash)
    {
        my ($pvals, $bvals) = HAMS->getAttributes("/var/opt/hams/hamsdef");
        my $primary   = $$pvals{'hostname'};
        my $backup    = $$bvals{'hostname'};
        my $active_ms = HAMS->determineActive($pvals, $bvals);
        my $inactive_ms;
        if ($active_ms eq $primary) { $inactive_ms = $backup; }
        else { $inactive_ms = $primary; }
        my $lsrsrc_cmd =
          "CT_MANAGEMENT_SCOPE=2 /usr/bin/lsrsrc-api -D ':|:' -i -s IBM.ManagedNode::\"NodeNameList IN ('$inactive_ms')\"::Hostname::ManagementServer";
        my @values = NodeUtils->runcmd($lsrsrc_cmd, -1);

        foreach my $line (@values)
        {
            my ($hostname, $ms) = split ':\|:', $line;
            $HAMS::MSHash{$hostname} = $ms;
        }
    }
    return $HAMS::MSHash{$node};
}

sub getInactiveRSCTHostname
{
    if (defined $HAMS::LSNODEID) { return $HAMS::LSNODEID; }
    my ($pvals, $bvals) = HAMS->getAttributes("/var/opt/hams/hamsdef");
    my $inactive_ms = HAMS->determineInactive($pvals, $bvals);
    my $output =
      NodeUtils->runcmd(
        "$::DSH -t $HAMS::TIMEOUT -vn $inactive_ms /usr/sbin/rsct/bin/lsnodeid",
        -1
        );
    my ($hostname, $nodeid) = split ": ", $output;
    $HAMS::LSNODEID = $nodeid;    #cache for later
    return $nodeid;
}

sub checkHAMSInstalled
{

    my ($class, $primary, $backup) = @_;

    my $cmd;
    my $isAIX = 0;
    if (NodeUtils->isAIX())
    {
        $cmd   = "$::LSLPP -cLq csm.hams";
        $isAIX = 1;
    }
    else
    {    #Linux
        $cmd = "$::RPMCMD -q csm.hams";
    }

    my @output =
      NodeUtils->runcmd(
                       "$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup \"$cmd\"",
                       -1);
    my %installed;

    if ($isAIX)
    {
        foreach my $line (@output)
        {
            my ($hostname, $name1, $name2, $version) = split(/:/, $line, 4);
            if ($name1 =~ "csm.hams")
            {
                $installed{$hostname} = $version;
            }
        }
    }
    else
    {
        foreach my $line (@output)
        {
            my ($hostname, $rpm) = split(/:/, $line, 2);
            if ($rpm =~ "csm.hams-")
            {
                my ($name, $version, $spin) = split(/-/, $rpm);
                $installed{$hostname} = $version;
            }
        }

    }
    my @notinstalled;
    foreach my $node ($primary, $backup)
    {
        if (!$installed{$node})
        {
            push @notinstalled, $node;
        }
    }
    if (scalar @notinstalled > 0)
    {
        my $nstring = join ",", @notinstalled;
        MessageUtils->message('I', "IMsgNotInstalled", $nstring);
        exit 1;

        #TODO: change to error message
    }
}

sub waitRG
{
    my ($class, $state, $primary, $backup) = @_;
    my $rg = "Active_MS Inactive_MS";
    my @output;
    if ($state == 1)
    {
        MessageUtils->message('I', 'IMsgWaitingRG', $rg, "online");
    }
    elsif ($state == 2)
    {
        MessageUtils->message('I', 'IMsgWaitingRG', $rg, "offline");
    }
    my $count = 1;
    my $active_ms;
    my $inactive_ms;
    my $timeout = 5 * $HAMS::LONGTIME;
    my $ver     = $::VERBOSE;
    my @numnodes =
      NodeUtils->runcmd(
         "CT_MANAGEMENT_SCOPE=1 /usr/bin/lsrsrc-api -s IBM.ManagedNode::::Name",
         -1
         );
    if ($#numnodes > 100) { $timeout = 2 * $#numnodes; }

    while (   (($active_ms != $state) || ($inactive_ms != $state))
           && ($count < $timeout))
    {
        sleep $HAMS::INTERVAL;
        if (!$::VERBOSE)
        {
            print ".";
        }
        @output    =
          NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=2 /usr/bin/lsrsrc-api -i -s IBM.ResourceGroup::\"Name IN ('Active_MS','Inactive_MS')\"::Name::OpState 2>&1",
            -1
            );
        $::VERBOSE = $ver;
        if ($::RUNCMD_RC != 0)
        {
            $active_ms = $inactive_ms = $state;
        }
        else
        {
            chomp @output;
            foreach my $line (@output)
            {
                my ($name, $opstate) = split '::', $line;
                if ($opstate == $state)
                {
                    if    ($name eq "Active_MS")   { $active_ms   = $state; }
                    elsif ($name eq "Inactive_MS") { $inactive_ms = $state; }
                }
            }
        }
        $count++;
    }
    print "\n";
    if ($count == $timeout)
    {    #timed out
        MessageUtils->message('E2', 'EMsgTimedOut', "Active_MS, Inactive_MS");
    }
    if ($state == 1)
    {
        MessageUtils->message('I', 'IMsgHAMSStarted');
    }
    elsif ($state == 2)
    {
        MessageUtils->message('I', 'IMsgHAMSStopped');
    }
    return 0;    #successful
}

sub waitActive_MS
{
    my ($class, $state, $primary, $backup) = @_;

    my $rg = "Active_MS";
    if ($state == 1)
    {
        MessageUtils->message('I', 'IMsgWaitingRG', $rg, "online");
    }
    else
    {
        MessageUtils->message('I', 'IMsgWaitingRG', $rg, "offline");
    }

    my @output;
    my $count = 1;
    my $active_ms;
    my $times = 2 * $HAMS::LONGTIME / $HAMS::SHORTINTER;
    while (($active_ms != $state) && ($count <= $times))
    {
        print ".";
        sleep $HAMS::SHORTINTER;
        @output =
          NodeUtils->runcmd("CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/lsrsrc-api -i -s IBM.ResourceGroup::\"Name IN ('Active_MS')\"::Name::OpState", 0);
        chomp @output;
        foreach my $line (@output)
        {
            my ($name, $opstate) = split '::', $line;
            if ($opstate == $state)
            {
                if ($name eq "Active_MS") { $active_ms = $state; }
            }
        }
        $count++;
    }
    print "\n";
    if ($count >= $times)
    {    #timed out
        MessageUtils->message('E2', 'EMsgTimedOut', "Active_MS, Inactive_MS");
    }

    return 0;    #successful
}

sub removeCHARMRes
{
    my ($class, $remote) = @_;

    # remove the resource groups, service IP and equivalency in one command
    NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/rmrsrc-api -i -s IBM.ResourceGroup::\"Name IN ('Active_MS','Inactive_MS')\" -s IBM.ServiceIP::\"Name IN ('HAMS_IPAT')\" -i -s IBM.Equivalency::\"Name IN ('HAMS_NET_EQU')\"",
        -1
        );

    HAMS->rmrsrc_apps($remote);

    #make sure the resources were removed
    my @resources =
      qw(Active_CSM Inactive_CSM Active_CSM_Dep Active_CSM_Req Shared_Disk_Dep Shared_Disk Shared_Disk_Req Save_CSM_Data Public_IP_Dep Public_IP_Req Gather_CSM_Nodelist Active_VLAN_Connection Inactive_VLAN_Connection);
    my $where = NodeUtils->buildWhereInList(\@resources, "Name");
    my @res_defined =
      NodeUtils->runcmd(
                       "/usr/bin/lsrsrc-api -s IBM.Application::${where}::Name",
                       -1);
    if (!$::RUNCMD_RC && (scalar @res_defined > 0))
    {    #there are some resources left
            #my $string = join " ", @res_defined;
            # buildWhereInList->message('E2', 'EMsgResourcesLeft', $string);
        sleep $HAMS::INTERVAL;
        HAMS->rmrsrc_apps($remote);
        NodeUtils->runcmd("/usr/sbin/rsct/bin/dms/stopdms -s cthats", 0);
    }

    NodeUtils->runcmd(
                   "/usr/bin/rmrsrc -s \"Name IN ('HAMS_IPAT')\" IBM.ServiceIP",
                   -1);
    NodeUtils->runcmd(
        "/usr/sbin/rsct/bin/chrsrc -c IBM.PeerNode OpQuorumTieBreaker=\"Operator\"",
        -1
        );
    NodeUtils->runcmd(
                 "/usr/bin/rmrsrc -s \"Name IN ('$::TB_NAME')\" IBM.TieBreaker",
                 -1);
}

#1 = force copy
sub copyLocal
{
    my ($class, $force, $pvals, $bvals, $cvals) = @_;
    if (NodeUtils->isLinux())
    {
        my $last_version = HAMS->getLastVersion();
        my $csmversion   = NodeUtils->get_CSMVersion("csm.server");
        if (
            ($force)
            || (   (defined $last_version)
                && ($last_version ne "")
                && ($last_version ne $csmversion))
          )
        {
            my $distrib = NodeUtils->get_DistributionName();

            # my $version = NodeUtils->get_DistributionVersion();
            NodeUtils->runcmd("$::UMOUNT /csminstall");
            my $dir = "/csminstall/Linux/$distrib/csm/$csmversion/packages/";
            if (-d $dir)
            {
                NodeUtils->runcmd(
                    "$::CP -a $dir /var/opt/hams/mnt/csminstall/Linux/$distrib/csm/$csmversion/"
                    );
            }
            NodeUtils->runcmd(
                    "$::MOUNT --bind /var/opt/hams/mnt/csminstall /csminstall");
        }
    }
}

sub storeLastVersion
{

    my $csmversion = NodeUtils->get_CSMVersion("csm.server");
    NodeUtils->runcmd("echo $csmversion > /var/opt/hams/mnt/.last_csmversion");
}

sub getLastVersion
{
    my $file = "/var/opt/hams/mnt/.last_csmversion";
    if (-e $file)
    {
        open(FILE, "<$file")
          or MessageUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
        my @contents = <FILE>;
        close FILE;
        my $csmversion = $contents[0];
        $csmversion =~ s/^\s*//;    # Remove any leading whitespace
        $csmversion =~ s/\s*$//;    # Remove any trailing whitespace
        return $csmversion;
    }
    return "";
}

#log events via CLOG

sub logEvent
{
    my ($class, $sccsID, $message, $fields) = @_;
    if (!defined $::clog_ptr)
    {
        $::clog_ptr = CLOG->new(
                                compID         => "_HAM",
                                lppID          => "HAMS",
                                messageCatalog => "hams.cat",
                                messageSet     => "hams",
                                messageMapDir  => "/opt/csm/msgmaps"
                                );
    }

    my $return_code =
      $::clog_ptr->clog_rec(
                            fileName    => __FILE__,
                            lineNumber  => __LINE__,
                            sccsID      => $sccsID,
                            messageType => CLOG::CLOG_INFO_MSG,
                            messageID   => $message,
                            fields      => $fields
                            );
    if ($return_code != 0)
    {
        MessageUtils->message('E', 'EMsgCLOGError', $message, $return_code);
    }
}

sub resource_started
{
    my ($class, $resource) = @_;
    my $message = "IMsgStartedResource";
    MessageUtils->message("LI", $message, $resource);
    my $sccsID = "1.363";
    my @fields = ("$resource");
    HAMS->logEvent($sccsID, $message, \@fields);

}

sub resource_stopped
{
    my ($class, $resource) = @_;
    my $message = "IMsgStoppedResource";
    MessageUtils->message("LI", $message, $resource);
    my $sccsID = "1.363";
    my @fields = ("$resource");
    HAMS->logEvent($sccsID, $message, \@fields);
}

sub start_hwctrlrm
{
    shift @_;    #class
    my $nodes = join ",", @_;
    my $cmd;
    if ($nodes !~ m/\w/)
    {            #run on localhost
        $cmd = "/opt/csm/csmbin/hams/hwctrl --start";
    }
    else
    {            #run through dsh
        $cmd =
          "$::DSH -t $HAMS::TIMEOUT -vn $nodes /opt/csm/csmbin/hams/hwctrl --start";
    }
    NodeUtils->runcmd($cmd, 0);
    if ($::RUNCMD_RC) { return 1; }
    return 0;
}

sub stop_hwctrlrm
{
    shift @_;    #class
    my $nodes = join ",", @_;
    my $cmd;
    if ($nodes !~ m/\w/)
    {            #run on localhost
        $cmd = "/opt/csm/csmbin/hams/hwctrl --stop";
    }
    else
    {            #run through dsh
        $cmd =
          "$::DSH -t $HAMS::MIDTIME -vn $nodes /opt/csm/csmbin/hams/hwctrl --stop";
    }
    NodeUtils->runcmd($cmd, 0);
    if ($::RUNCMD_RC) { return 1; }
    return 0;
}

sub runact_hwctrlrm
{

    my $rc = NetworkUtils->verify_atd;
    if ($rc && NodeUtils->isLinux())
    {
        NodeUtils->runcmd(
	    "CT_MANAGEMENT_SCOPE=1 /usr/bin/runact-api -c IBM.HwCtrlPoint::::HAModeActive::NewHAMode::1", 0
            );
    }
    else
    {
        NodeUtils->runcmd(
            "/bin/echo \"CT_MANAGEMENT_SCOPE=1 /usr/bin/runact-api -c IBM.HwCtrlPoint::::HAModeActive::NewHAMode::1\" | at now", 0
            );
    }

}

sub rmrsrc_apps
{
    my ($class, $remote) = @_;
    my $cmd =
      "/opt/csm/csmbin/hams/rmapplications Active_CSM Inactive_CSM Active_CSM_Dep Active_CSM_Req Shared_Disk_Dep Shared_Disk Shared_Disk_Req Save_CSM_Data Public_IP_Dep Public_IP_Req Gather_CSM_Nodelist Active_VLAN_Connection Inactive_VLAN_Connection";
    NodeUtils->runcmd($cmd, -1);

    $cmd =
      "$::DSH -t $HAMS::LONGTIME -vn $remote \"/opt/csm/csmbin/hams/rmapplications Active_CSM Inactive_CSM Active_CSM_Dep Active_CSM_Req Shared_Disk_Dep Shared_Disk Shared_Disk_Req Save_CSM_Data Public_IP_Dep Public_IP_Req Gather_CSM_Nodelist Active_VLAN_Connection Inactive_VLAN_Connection \"";
    NodeUtils->runcmd($cmd, -1);
}
#-----------------------------------------------------------------------------
# Runs on the Inactive Management Server
# If the license has not already been accepted then
# this gets a copy of the license file from the active MS
#
#
#-----------------------------------------------------------------------------
sub accept_csm_license
{
    my ($class, $pvals, $bvals, $cvals) = @_;

    #first see if we can already list the nodes
    my $need_license = HAMS->check_dmsrm();
    if (!$need_license) { return 0; }

    #otherwise we need to accept the license
    my $active_ms
      ;    #can't get active_ms from resources since they aren't online yet
    if ($$cvals{'hostname'} eq $$pvals{'hostname'})
    {
        $active_ms = $$bvals{'hostname'};
    }
    else { $active_ms = $$pvals{'hostname'}; }

    if ($active_ms && (-e "/var/opt/csm"))
    {

        my $found = 0;
        my $count = 0;
        my $times = 48; # up to 4 minutes, for license file to become available
        while (($found == 0) && ($count < $times))
        {
            my $output =
              NodeUtils->runcmd(
                 "LANG=C $::REMOTE_SHELL $active_ms \"$::LS  /var/opt/csm/lic/csmlum.tab\"");
            HAMS->printDebug("$output \n");
            if (($output =~ /csmlum.tab/) && ($output !~ /not found/))
            {
                $found = 1;
            }
            else
            {
                sleep $HAMS::MIDINTER;
                $count++;
            }
        }
        $count = 0;
        if ($found)
        {
            while ($need_license && ($count < $times))
            {
                my $copying = 1;
                my $cmd;
                eval {
                    alarm(50);
                    $cmd =
                          "$::REMOTECOPYCMD -rp $active_ms:/var/opt/csm/* /var/opt/csm";
                    NodeUtils->runcmd($cmd, -1);
                    $copying = 0;
                    alarm(0);
                };
                if ($copying)
                {
                    MessageUtils->message('E', 'EMsgDcpTimeout', $cmd);
                }

                #now set the correct LicenseProductVersion
                my $licdir  = "/var/opt/csm/lic";
                my $lumfile = 'csmlum.lic';
                if (!-e "$licdir/$lumfile")
                {
                    $lumfile = 'csmlum.tab';
                }
                $lumfile = "$licdir/$lumfile";
                my $lumversion =
                  NodeUtils->runcmd("/bin/grep ProductVersion $lumfile", 0);
                if ($::RUNCMD_RC == 0)
                {
                    $lumversion =~ s/ProductVersion=//;
                    my $origenv = $ENV{'CT_MANAGEMENT_SCOPE'};
                    $ENV{'CT_MANAGEMENT_SCOPE'} = 1;
                    NodeUtils->runrmccmd('chrsrc-api', '-i',
                        qq(-s IBM.DmsCtrl::::LicenseProductVersion::$lumversion)
                        );
                    $ENV{'CT_MANAGEMENT_SCOPE'} = $origenv;
                }
                NodeUtils->runcmd(
                     "CT_MANAGEMENT_SCOPE=1 /usr/bin/refrsrc-api -c IBM.DmsCtrl"
                     );

                #NodeUtils->runrmccmd('refrsrc-api', '-c IBM.DmsCtrl');
                $need_license =
                  HAMS->check_dmsrm();    #check to see its working now
                $count++;
            }
        }

    }

    if (!$need_license) { return 0; }
    else { return 1; }                    #could not accept license

}

sub check_dmsrm
{
    NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=1 /usr/sbin/rsct/bin/lsrsrc-api -D ':|:' -i -s IBM.ManagedNode::::Hostname",
        -1
        );
    if (!$::RUNCMD_RC) { return 0; }      #can already list the nodes
    else { return $::RUNCMD_RC; }
}

#logs the monitor exit code through clog if a file exists
sub monitor_debug
{
    my ($class, $resource, $exit_code) = @_;
    if (-e "/var/opt/hams/resources/debug"
        || ($exit_code != 1 && $exit_code != 2 && $exit_code != 0))
    {
        my $message = "IMsgMonitoredResource";
        MessageUtils->message("lV", $message, $resource, $exit_code)
          ;    #display only -- do not log
        my $sccsID = "1.363";
        my @fields = ("$resource", $exit_code);
        HAMS->logEvent($sccsID, $message, \@fields);
        $? = $exit_code;    #make sure this wasn't changed by CLOG
    }
}

#usage HAMS->get_hdisk($$cvals{'VolumeGroup'});
#AIX only
sub get_hdisk
{
    my ($class, $vg) = @_;
    my @disks;

    my $vid    = NodeUtils->runcmd("/usr/sbin/getlvodm -v $vg");
    my @output = NodeUtils->runcmd("/usr/sbin/getlvodm -w $vid");
    foreach my $line (@output)
    {
        my ($id, $hdisk) = split " ", $line;
        if ($hdisk =~ m/\S+/)
        {
            push @disks, "/dev/$hdisk";
        }
    }
    return @disks;
}

#usage: HAMS->break_reservation($hdisk);
#AIX only
sub break_reservation
{
    my ($class, @disks) = @_;
    foreach my $disk (@disks)
    {
        my ($nothing, $diskname) = split "\/dev\/", $disk;

        #determine disk type, code taken from HACMP
        my $parent =
          NodeUtils->runcmd("/usr/sbin/lsdev -Cc disk -l $diskname -F parent");
        my $pddvln =
          NodeUtils->runcmd("/usr/sbin/lsdev -Cc disk -l $diskname -F PdDvLn");
        my $disktype;
        if ($parent != m/\w/)
        {
            MessageUtils->message('E', 'EMsgFailedToGetDiskType', $disk);
        }
        if    ($parent =~ m/scsi/) { $disktype = "SCSIDISK"; }
        elsif ($parent =~ m/ssar/) { $disktype = "SSA"; }
        else
        {
            NodeUtils->runcmd(
                "/bin/odmget -q \"name = $parent AND PdDvLn = disk/fdar/array\" CuDv",
                -1
                );
            if (!$::RUNCMD_RC)
            {

                #   Either Dolphin or Flute.  They behave differently, but have
                #   nearly identical ODM entries.
                my $daclist =
                  NodeUtils->runcmd(
                    "/bin/odmget -q \"name = $parent AND attribute = all_controller\" CuAt | grep 'value =' | cut -f2 -d'\"'"
                    );
                if ($daclist !~ m/\w/)
                {
                    $disktype = "FCPARRAY";
                }    #daclist is empty
                else { $disktype = "FLUTE"; }
            }
            elsif ($parent =~ m/dar/)   { $disktype = "ARRAY"; }
            elsif ($parent =~ m/dpo/)   { $disktype = "DPO"; }
            elsif ($parent =~ m/fscsi/) { $disktype = "FSCSI"; }
        }

        $ENV{'VERBOSE_LOGGING'} = "high";

        if ($disktype eq "FLUTE")
        {
            NodeUtils->runcmd("/opt/csm/csmbin/hams/cl_flutereset $disk");
        }
        elsif ($disktype eq "SSA")
        {
            NodeUtils->runcmd("/opt/csm/csmbin/hams/cl_scdiskrsrv $disk");
        }
        else
        {
            MessageUtils->message('E', 'EMsgDiskTypeNotSupported', $disktype);
        }
    }
}

sub try_write_shared_disk
{
    my ($class, $pvals, $bvals, $cvals);
    my $pid;

    #check to see if an old child process is already running, if so don't fork, just return 3
    my @processes = NodeUtils->runcmd("/bin/ps -eo pid,args", -1);
    my @checkprocs = grep(/hamscheckdisk/, @processes);
    if (scalar @checkprocs > 0)
    {
        HAMS->printDebug("grep is true: @checkprocs");
        $::TOUCH_FAILED = 1;
        exit 3;
    }

    #try writing to the shared disk -- set an alarm so we don't wait forever
    $SIG{ALRM} = sub { die "timeout expired\n" };
    eval {
        alarm(35);
        $::TOUCH_FAILED = 1;
        HAMS->printDebug("spawing child\n");
        $pid = fork();
        if (!defined $pid)
        {
            undef $::TOUCH_FAILED;
            MessageUtils->message('E1', 'EMsgCantFork');
        }
        if ($pid == 0)
        {

            # Child - do some work
            undef $::TOUCH_FAILED;
            NodeUtils->runcmd("/opt/csm/csmbin/hams/hamscheckdisk");
            exit 0;
        }
        HAMS->printDebug("Waiting for child\n");
        my $child = waitpid(-1, 0);    # Wait for any child
        my $status = $? >> 8;
        HAMS->printDebug("Parent reaped $child, returned $status\n");
        if ($status == 3)
        {
            $::TOUCH_FAILED = 1;
            alarm(0);
            exit 3;
        }
        undef $::TOUCH_FAILED;         #looks like the shared disk is reachable
        alarm(0);
    };
    kill 9, $pid;

    # If the message from the eval is from the timeout subroutine,
    # we could not touch the file in the given amount of time
    # Write a message and exit.
    #
    if ($@ =~ /timeout expired/) { $::TOUCH_FAILED = 1; exit 3; }
    delete $SIG{ALRM};
}

sub displayLicenseAgreement
{
    my $tab = shift;
    my $rc  = HAMS->invokeLAP();
    if ($rc == 9)
    {

        # Tell the user the license was accepted.
        # a return code of 9 from invokeLAP means
        # that the user accepted it.
        # MessageUtils->msg($::MSGS, 'I', 'EMsgACCEPTED');
        MessageUtils->message('I', 'IMsgAccepted');
    }
    else
    {

        # Tell the user the license was not accepted.
        # MessageUtils->msg($::MSGS, 'W3', 'EMsgNOT_ACCEPTED');
        MessageUtils->message('W3', 'EMsgNOT_Accepted');
    }

    # return true or false if the license was accepted or not.
    return ($rc == 9);
}

#--------------------------------------------------------------------------------

=head3    invokeLAP

        Invoke the LAP tool and display the CSM license agreements.
        The LAP tool will be invoked and copy the LA and LI files to

            /var/opt/csm/<VRnumber>/<type>/license/<lang>.txt
        Where:
                <VRnumber> = CSM Version (1.3, etc)
                <type>     = tryandbuy or full, depending upon the license.
                <lang>     = one of the 17 supported LAP languages.

        Arguments:
                A string - either: "tab" or "full"

        Returns: one of [ 3 | 9 | 127 ]
                If everything ran successful, then LAP will return 9,
        otherwise, if the user returns 3, the license was not
        accepted.  If something went wrong with the LAP tool,
        then an error of 127 will be returned.
        Error:
                127

        Example: Doesn't appear to be used in csm/install

        Comments: none

=cut

#--------------------------------------------------------------------------------

sub invokeLAP
{
    my ($class, $licType) = @_;
    my ($csmVersion, $laDir, $strCmd, $rc);
    my $laptool = "/opt/csm/install/lapapp.jar";

    # make sure that $licType is either tab or full
    $licType = "full";

    # get the CSM version:
    #$csmVersion = NodeUtils->get_CSMVersion("csm.server");
    #$csmVersion = sprintf "%.1f", $csmVersion;                  # e.g. "1.3"

    $csmVersion = "1.5";

    # check that the LA and LI files are in place:
    if (NodeUtils->isLinux())
    {
        my $arch = ArchiveUtils->get_PkgArchitecture();
        if ($arch =~ m/ppc/)
        {    #pLinux
            $laDir = "/opt/csm/hams/$csmVersion/pLinux/license/$licType";
        }
        else
        {    #xLinux
            $laDir = "/opt/csm/hams/$csmVersion/xLinux/license/$licType";
        }
    }
    else     # AIX
    {
        $laDir = "/opt/csm/hams/$csmVersion/AIX/license/$licType";
    }

    unless (-f "$laDir/LA_en")
    {
        MessageUtils->messageFromCat(
                                     'csmInstall.cat',     $::MSGMAPPATH,
                                     'csminstall',         'E127',
                                     'EMsgCANT_READ_FILE', "$laDir/LA_en"
                                     );
    }

    # check that the LAP tool is in place:
    unless (-f "$laptool")
    {
        MessageUtils->messageFromCat(
                                     'csmInstall.cat',     $::MSGMAPPATH,
                                     'csminstall',         'E127',
                                     'EMsgCANT_READ_FILE', $laptool
                                     );
    }

    # Everything should be in place now, so let's call LAP and  construct the command:

    if (NodeUtils->isLinux())
    {

        #try to find java
        my $found_java = 0;
        my ($java, $rt, $i18n);
        my @output = NodeUtils->runcmd("$::LS -1d /opt/IBMJava\*");
        while (($found_java != 1) && (@output))
        {
            my $dir = shift @output;
            $dir =~ s/^\s*//;    # Remove any leading whitespace
            $dir =~ s/\s*$//;    # Remove any trailing whitespace
            my $tmp_java = "$dir/jre/bin/java";
            my $tmp_rt   = "$dir/jre/lib/rt.jar";
            my $tmp_i18n = "$dir/jre/lib/i18n.jar";
            if (-e "$tmp_java")
            {
                $found_java = 1;
                $java       = $tmp_java;
                $rt         = $tmp_rt;
                $i18n       = $tmp_i18n;
            }
        }
        if ($found_java)
        {
            $strCmd = "$java -cp /opt/csm/install/lapapp.jar:${rt}:${i18n}:";
        }
        else
        {    #try the old way
                # check that Java is installed... and use the right one.
            if (-d "/opt/IBMJava2-141")
            {
                $strCmd = "/opt/IBMJava2-141/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-141/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-141/jre/lib/i18n.jar:";
            }
            elsif (-d "/opt/IBMJava2-131")
            {
                $strCmd = "/opt/IBMJava2-131/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-131/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-131/jre/lib/i18n.jar:";
            }
            elsif (-d "/opt/IBMJava2-13")
            {
                $strCmd = "/opt/IBMJava2-13/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-13/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-13/jre/lib/i18n.jar:";
            }
            elsif (-d "/opt/IBMJava2-ppc-141")
            {
                $strCmd = "/opt/IBMJava2-ppc-141/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-ppc-141/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-ppc-141/jre/lib/i18n.jar:";
            }
            elsif (-d "/opt/IBMJava2-ppc-131")
            {
                $strCmd = "/opt/IBMJava2-ppc-131/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-ppc-131/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-ppc-131/jre/lib/i18n.jar:";
            }
            elsif (-d "/usr/lib/IBMJava2-1.3.1")
            {
                $strCmd = "/usr/lib/IBMJava2-1.3.1/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/usr/lib/IBMJava2-1.3.1/jre/lib/rt.jar:";
                $strCmd .= "/usr/lib/IBMJava2-1.3.1/jre/lib/i18n.jar:";
            }
            else    # java isn't installed maybe
            {
                my $string = "IBMJava2-13[1]";
                MessageUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH,
                           'csminstall', 'E127', 'EMsgMissingPackage', $string);
            }
        }
    }
    else
    {               # AIX
                    #try to find java
        my $found_java = 0;
        my ($java, $rt, $i18n);
        my @output = NodeUtils->runcmd("$::LS -1d /usr/java\*");
        while (($found_java != 1) && (@output))
        {
            my $dir = shift @output;
            $dir =~ s/^\s*//;    # Remove any leading whitespace
            $dir =~ s/\s*$//;    # Remove any trailing whitespace
            my $tmp_java = "$dir/jre/bin/java";
            my $tmp_rt   = "$dir/jre/lib/rt.jar";
            my $tmp_i18n = "$dir/jre/lib/i18n.jar";
            if (-e "$tmp_java")
            {
                $found_java = 1;
                $java       = $tmp_java;
                $rt         = $tmp_rt;
                $i18n       = $tmp_i18n;
            }
        }
        if ($found_java)
        {
            $strCmd = "$java -cp /opt/csm/install/lapapp.jar:${rt}:${i18n}:";
        }
        else
        {
            if (-d "/usr/java141")
            {
                $strCmd = "/usr/java141/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/usr/java141/jre/lib/rt.jar:";
                $strCmd .= "/usr/java141/jre/lib/i18n.jar:";
            }
            else
            {
                $strCmd = "/usr/java131/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/usr/java131/jre/lib/rt.jar:";
                $strCmd .= "/usr/java131/jre/lib/i18n.jar:";
            }
        }
    }

    # this part of the command is common in both AIX and Linux:
    $strCmd .= " com.ibm.lex.lapapp.LAP ";
    $strCmd .= "-l $laDir ";
    $strCmd .= "-s /var/opt/hams/$csmVersion/$licType ";
    $strCmd .= "-text_only";
    $rc = system("$strCmd");
    $rc = $rc >> 8;

    # kill processes if they exist:
    if (NodeUtils->isLinux)
    {
        my @output = NodeUtils->runcmd("ps -efl --columns 300 | grep lapapp | awk '{print \$4}'", 0);
        foreach my $pid (@output)
        {
            if ($pid =~ m/\d+/)
            {
                NodeUtils->runcmd("kill -9 $pid", -1);
            }
        }
    }

    return $rc;
}

#check to see if there are any symlinked files in /cfmroot
sub check_cfm_links
{
    my ($class, $path, $silent) = @_;
    if (!defined $path)
    {
        $path = "/cfmroot";
    }
    my %badlinks;
    my $file;
    if (!-d $path) { return; }
    my $dir;
    opendir($dir, $path)
      or MessageUtils->message('E2', 'EMsgPerlFailed', "opendir", $path, $!);
    while ($file = readdir($dir))
    {
        my $wholename = join '/', $path, $file;
        if ($file ne '.' and $file ne '..')
        {
            if (-l $wholename)
            {    #if its a link
                my $link = readlink($wholename);
                if ($link !~ m/cfmroot/)
                {
                    $badlinks{$wholename} = $link;
                }
            }
            elsif (-d $wholename)
            {
                my $child = HAMS->check_cfm_links($wholename, 1);
                map { $badlinks{$_} = $child->{$_} } keys %{$child};
            }
        }
    }
    closedir($dir)
      or MessageUtils->message('E2', 'EMsgPerlFailed', "closedir", $path, $!);
    if (!defined $silent)
    {
        my @files = keys %badlinks;
        if (scalar @files > 0)
        {
            MessageUtils->message('E', 'EMsgCFMSymLink');
            foreach my $f (@files)
            {
                MessageUtils->message('E', 'IMsgLink', $f);
            }
        }
    }
    return (\%badlinks);
}

#check the validity of the shared disk
sub check_disk
{
    my ($class, $cvals) = @_;
    if (NodeUtils->isLinux())
    {

        #first see if its mounted, if it is return
        if (HAMS->isMountedonCurrent($cvals, $::MOUNT_POINT))
        {
            return 0;
        }
        my $disk = $$cvals{'PartitionDeviceName'};
        my $type = $$cvals{'FileSystemType'};
        my $cmd  = "/sbin/fsck -a -t $type $disk";

        my $distri = NodeUtils->get_DistributionName();

        # in the case of AIX, the fisrt function will return undef
        my $distri_ver = NodeUtils->get_DistributionVersion();
        if ($distri eq "SLES" && $distri_ver eq "9")
        {
            $cmd = "/sbin/fsck -f -a -t $type $disk";
        }

        my $output = NodeUtils->runcmd($cmd, 0);
        my $fsck_error = $::RUNCMD_RC;
        if ($fsck_error != 0)
        {
            MessageUtils->message('E', 'EMsgFsckError', $cmd, $output);
        }
    }
}

sub mkrg_mkrel
{
    my ($class, $cvals, $rvals) = @_;

    #make resource groups
    #Inactive_MS IBM.ResourceGroup Contains the names of all the resources that are required on the inactive management server
    #Active_MS IBM.ResourceGroup Contains the names of all the resources that are required on the active management server
    NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/mkrsrc-api IBM.ResourceGroup::MemberLocation::1::Priority::\"5\"::Name::\"Active_MS\"  IBM.ResourceGroup::MemberLocation::1::Name::\"Inactive_MS\""
        );

    if ($::IPAT == 1)
    {

        #make IBM.ServiceIP resources
        my $nodestring = "\"{'$$cvals{'hostname'}','$$rvals{'hostname'}'}\"";
        NodeUtils->runcmd(
            "CT_MANAGEMENT_SCOPE=2 /usr/bin/mkrsrc IBM.ServiceIP Name=\"HAMS_IPAT\" NodeNameList=$nodestring IPAddress=$$cvals{'PublicIPAddress'} NetMask=$$cvals{'PublicIPNetmask'} ProtectionMode=0"
            );
        NodeUtils->runcmd(
            "/usr/bin/mkequ HAMS_NET_EQU IBM.NetworkInterface:$$cvals{'PublicIPAdapter'}:$$cvals{'hostname'},$$rvals{'PublicIPAdapter'}:$$rvals{'hostname'}"
            );
        NodeUtils->runcmd(
            "/usr/bin/addrgmbr -g Active_MS IBM.ServiceIP:HAMS_IPAT IBM.Application:Public_IP_Dep IBM.Application:Public_IP_Req "
            );
        NodeUtils->runcmd(
            "/usr/bin/mkrel -p DependsOn -G IBM.Equivalency:HAMS_NET_EQU -S IBM.ServiceIP:HAMS_IPAT HAMS_IPAT_DependsOn"
            );
    }

    NodeUtils->runcmd(
        "/usr/bin/addrgmbr -g Active_MS IBM.Application:Active_CSM IBM.Application:Active_CSM_Dep IBM.Application:Active_CSM_Req IBM.Application:Shared_Disk_Dep IBM.Application:Shared_Disk IBM.Application:Shared_Disk_Req IBM.Application:Save_CSM_Data IBM.Application:Active_VLAN_Connection"
        );

    NodeUtils->runcmd(
        "/usr/bin/addrgmbr -g Inactive_MS IBM.Application:Inactive_CSM IBM.Application:Gather_CSM_Nodelist IBM.Application:Inactive_VLAN_Connection"
        );

    #AC and IC relationships
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p Collocated -o IfNotOffline -S IBM.Application:Active_CSM -G IBM.Application:Inactive_CSM  HAMS_ACIC_Collocated"
        );
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p AntiCollocated -G IBM.Application:Shared_Disk -S  IBM.Application:Inactive_CSM HAMS_ICAC_Anti_Collocated"
        );

    #AC relationships inside Active_MS
    #DependsOn
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p DependsOn -G IBM.Application:Shared_Disk -S IBM.Application:Active_CSM HAMS_ACSD_DependsOn"
        );
    if ($::IPAT)
    {
        NodeUtils->runcmd(
            "/usr/bin/mkrel -p DependsOn -G IBM.ServiceIP:HAMS_IPAT -S IBM.Application:Active_CSM HAMS_ACPI_DependsOn"
            );
    }

    NodeUtils->runcmd(
        "/usr/bin/mkrel -p DependsOn -G IBM.Application:Active_VLAN_Connection -S IBM.Application:Active_CSM HAMS_ACAVC_DependsOn"
        );
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p DependsOn -G IBM.Application:Active_CSM -S IBM.Application:Save_CSM_Data HAMS_SVDAC_DependsOn"
        );
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p DependsOn -G IBM.Application:Active_CSM_Dep -S IBM.Application:Active_CSM HAMS_ACACD_DependsOn"
        );
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p DependsOn -G IBM.Application:Active_CSM -S IBM.Application:Active_CSM_Req HAMS_ACRAC_DependsOn"
        );

    #Shared disk relationships
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p DependsOn -G IBM.Application:Shared_Disk_Dep -S IBM.Application:Shared_Disk HAMS_SDSDD_DependsOn"
        );
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p DependsOn -G IBM.Application:Shared_Disk -S IBM.Application:Shared_Disk_Req HAMS_SDRSD_DependsOn"
        );

    #Inactive CSM relationships
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p DependsOn -G IBM.Application:Inactive_CSM -S IBM.Application:Gather_CSM_Nodelist HAMS_GCNIC_DependsOn"
        );
    NodeUtils->runcmd(
        "/usr/bin/mkrel -p DependsOn -G IBM.Application:Inactive_VLAN_Connection -S IBM.Application:Inactive_CSM HAMS_ICIVC_DependsOn"
        );

    if ($::IPAT)
    {
        NodeUtils->runcmd(
            "/usr/bin/mkrel -p DependsOn -G IBM.Application:Public_IP_Dep -S IBM.ServiceIP:HAMS_IPAT HAMS_HIPID_DependsOn"
            );
        NodeUtils->runcmd(
            "/usr/bin/mkrel -p DependsOn -G IBM.ServiceIP:HAMS_IPAT -S IBM.Application:Public_IP_Req HAMS_PIRHI_DependsOn"
            );
    }
}

#get a lock so that only 1 csm command can be run at a time
sub block_csm_cmd
{
    my $file =
      "/var/opt/hams/resources/.blockCSMlock"
      ;    #lock a file in /var so it doesn't need to be removed
    open(BLOCKCSM, ">$file")
      or MessageUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);

    #use an alarm to timeout if we cannot get the lock in a reasonable amount of time
    my $cantlock = 0;
    eval {
        local $SIG{ALRM} = sub { die "alarm clock restart" };
        alarm 60;    # schedule alarm in 600 seconds
        eval {
            if (!flock(BLOCKCSM, LOCK_EX))
            {
                $cantlock = 1;
            }
        };
        alarm 0;     # cancel the alarm
    };
    alarm 0;         # race condition protection
    die if $@ && $@ !~ /alarm clock restart/;    # reraise
    if ($cantlock == 1) { die "cannot get lock on $file\n"; }

    return 0;
}

#release lock
sub unblock_csm_cmd
{
    close(BLOCKCSM);
}

#see if both MS are time synced
#return the time difference in seconds between two machines
sub checkTime
{
    my ($class, $primary, $backup) = @_;

    if (!defined $primary || !defined $backup)
    {
        return -1;
    }

    my $cmd = '"perl -e \'print time()\'"';
    $cmd = "$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup " . $cmd;
    my @output = NodeUtils->runcmd($cmd, -1);
    if ($::RUNCMD_RC)
    {
        MessageUtils->message('E', 'EMsgDshFailed');
        return -1;
    }

    my @times;
    foreach my $line (@output)
    {
        if ($line =~ /(.*):\s(\d*)/)
        {
            my $time = $2;
            push @times, $time;
        }
    }
    return abs($times[0] - $times[1]);
}

sub unexport
{
    my ($class, $dir) = @_;
    my @exported = NodeUtils->runcmd("$::EXPORTFS", -1);
    if (!$::RUNCMD_RC && (scalar @exported > 0))
    {

        #stuff is exported, look through it to see if we need to unexport anything
        foreach my $line (@exported)
        {
            if ($line =~ m/$dir/)
            {    #this is a shared disk directory
                if (NodeUtils->isAIX())
                {
                    NodeUtils->runcmd("$::EXPORTFS -i -u $line", 0);
                }
                else
                {
                    NodeUtils->runcmd("$::EXPORTFS -u \*:$line", 0);
                    # we have to run unportfs again to fix one Linux kernel bug
                    NodeUtils->runcmd("$::EXPORTFS -u \*:$line", 0);
                }
            }
        }
    }
}

sub offlineInactive_MS
{
    my ($class) = @_;

    NodeUtils->runcmd(
        "CT_MANAGEMENT_SCOPE=2 /usr/sbin/rsct/bin/chrsrc-api -e -i -o IBM.ResourceGroup::\"Inactive_MS\"::::NominalState::\"1\"",
        -1
        );

    my ($pvals, $bvals, $cvals) = HAMS->getAttributes($::HAMSDEF);
    while (HAMS->waitAppOffline($pvals, $bvals, "Inactive_CSM"))
    {
        sleep $HAMS::SHORTINTER;
    }
    while (HAMS->waitAppOffline($pvals, $bvals, "Gather_CSM_Nodelist"))
    {
        sleep $HAMS::SHORTINTER;
    }
    while (HAMS->waitAppOffline($pvals, $bvals, "Inactive_VLAN_Connection"))
    {
        sleep $HAMS::SHORTINTER;
    }

    unlink "/var/opt/hams/resources/Inactive_CSM";
}

sub remove_HAMS_links
{
    my @dirs = ("/etc/opt/csm", "/var/opt/csm", "/csminstall");
    foreach my $dir (@dirs)
    {
        if (-l $dir)
        {
            my $where = readlink("$dir");
            HAMS->printDebug("$dir is a symbolic link to $where \n");
            if ($where !~ /\.local/)
            {
                unlink("$dir");
                NodeUtils->runcmd("$::MKDIR -p $dir");
            }
        }
    }
}

#----------------------------------------------------------------

=head3 get_info

    get host, chan, id, lun and sequence number of shared disk on Linux

    Arguments:
      $partition: it is partition device name
      $msvals:        reference to the hash containing target MS info
      $inactive_vals: reference to the hash containing inactive MS info

    Return:
      a scalar that contains 4 disk attributes and sequence number (successful)
      1         (failed)

=cut

#----------------------------------------------------------------
sub get_info
{
    my ($class, $partition, $msvals, $inactive_vals) = @_;

    if (NodeUtils->isAIX())
    {
        return 1;
    }

    my $cmd = "/opt/csm/csmbin/hams/scsinum";
    my $info = NodeUtils->runcmd("$cmd $partition", 0);
    if ($::RUNCMD_RC != 0)
    {
        return 1;
    }
    my ($disk_name, $attrs) = split(' ', $info);
    $partition =~ s/\D+//g;
    $attrs .= " $partition";
    if (!defined $inactive_vals)
    {
        return "$$msvals{'hostname'}#$attrs";
    }
    else
    {
        my $first    = "$$msvals{'hostname'}#$attrs";
        my $i_part   = $$inactive_vals{'PartitionDeviceName'};
        my $hostname = $$inactive_vals{'hostname'};
        $info =
          NodeUtils->runcmd(
                          "$::DSH -t $HAMS::TIMEOUT -vn $hostname $cmd $i_part",
                          0);
        if ($::RUNCMD_RC != 0)
        {
            return 1;
        }
        my $dsh_hostname;
        ($dsh_hostname, $disk_name, $attrs) = split(' ', $info);
        $partition =~ s/\D+//g;
        $attrs .= " $partition";
        return "$first\n$$inactive_vals{'hostname'}#$attrs";
    }
}

#----------------------------------------------------------------

=head3 get_disk_name

    get disk name

    on Linux,
        it searched the file '/var/opt/hams/sharedDiskInfo' first.
        if it does not contain target disk name, this function will
        return $$ms{'PartitionDeviceName'}
    on AIX,
        it return $$ms{'PartitionDeviceName'}

    Arguments:
        $ms     reference to a hash that contains management server
        $chost  host name of local server

    Return;
        $disk   partition disk name

    Example:
        HAMS->get_disk_name($pvals, $current_hostname);

=cut

#-----------------------------------------------------------------
sub get_disk_name
{
    my ($class, $ms, $chost) = @_;
    my $disk = $$ms{'PartitionDeviceName'};
    my $file = "/var/opt/hams/sharedDiskInfo";

    if (NodeUtils->isAIX())
    {
        return $disk;
    }

    my ($isRemote, $info_file);
    if ($chost eq $$ms{'hostname'})
    {
        if (!-f $file)
        {
            return $disk;
        }
        $info_file = $file;
    }
    else
    {

        # if shared disk name of remote server is required,
        # copy it to local server and parse it
        $info_file = "/var/opt/hams/remote_sharedDiskInfo";
        my $dcp_cmd .= "$::REMOTECOPYCMD $$ms{'hostname'}\:$file $info_file";
        NodeUtils->runcmd($dcp_cmd, -1);
        if ($::RUNCMD_RC != 0)
        {
            return $disk;
        }
        $isRemote++;
    }

    # get disk name from the file if it exists
    open(FILE, "<$info_file")
      or MessageUtils->message('E2', 'EMsgPerlFailed', "open", $info_file, $!);
    my ($ms_name, $disk_info, $disk_name);
    while (my $line = <FILE>)
    {
        chomp $line;
        ($ms_name, $disk_info, $disk_name) = split('#', $line);
        if ($ms_name eq $$ms{'hostname'}) { last; }
    }
    close(FILE);
    if ($isRemote)
    {
        unlink $info_file;
    }
    if ($disk_name =~ m/\S+/g)
    {
        $disk_name =~ s/\s+//g;
        return $disk_name;
    }
    return $disk;
}

#-----------------------------------------------------------------

=head3 get_disk_by_info

    get device partition name of shared disk on Linux

    Arguments:
        $info:    a scalar that contains comma separated host, chan,
                  id, lun and sequence number of shared disk

    Return:
        a scalar that contains device partition name (when successful)
        undef   (when failed)

=cut

#-----------------------------------------------------------------
sub get_disk_by_info
{
    my ($class, $info) = @_;
    my (@disks);

    if (NodeUtils->isAIX())
    {
        return;
    }

    # list all disk partitions
    my $file = "/proc/partitions";
    open(FILE, "</proc/partitions")
      or MessageUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
    my @output = <FILE>;
    close FILE;
    foreach my $line (@output)
    {
        if ($line =~ /sd\D\d/g)
        {
            my $device = "\/dev\/";
            my ($major, $minor, $blocks, $disk) = split(' ', $line);
            $device .= $disk;
            push @disks, $device;
        }
    }
    if (!scalar(@disks))
    {
        return;
    }
    elsif (scalar(@disks) eq 1)
    {
        return $disks[0];
    }

    # select the partitions with suitable info
    my $cmd = "/opt/csm/csmbin/hams/scsinum ";
    my $disk_str = join(' ', @disks);
    my ($attrs, $seq_num) = split(' ', $info);
    @output = NodeUtils->runcmd("$cmd $disk_str", 0);
    foreach my $line (@output)
    {
        if ($line =~ $attrs)
        {
            my ($disk, $attr) = split(' ', $line);
            if ($disk =~ $seq_num)
            {
                return $disk;
            }
        }
    }
    return;    # if no available partition is found, return undef
}

#------------------------------------------------------------------

=head3 store_disk_name

    store local partition device name of shared disk on Linux
    this info will not be transfered to remote MS

    Arguments:
        $partition:   a scalar that contains partition device name
        $ms:          host name of target MS
        $inactive_ms: host name of inactive MS

    Return:

=cut

#------------------------------------------------------------------
sub store_disk_name
{
    my ($class, $partition, $ms, $inactive_ms) = @_;
    my $file = "/var/opt/hams/sharedDiskInfo";

    # get original contents first
    my @file_contents;
    open(FILE, "<$file")
      or MessageUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
    while (my $line = <FILE>) { push @file_contents, $line; }
    close(FILE);

    my @new_file_contents;
    foreach my $line (@file_contents)
    {
        chomp $line;
        my ($msname, $info, $disk_name) = split('#', $line);
        if ($msname eq $ms)
        {
            $line = "$ms#$info#$partition";
        }
        push @new_file_contents, "$line\n";
    }

    # update the files on both MS
    open(INFOFILE, ">$file")
      || MessageUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
    print INFOFILE @new_file_contents;
    close INFOFILE;
    my $dcp_cmd .= "$::REMOTECOPYCMD $file $inactive_ms\:$file";
    NodeUtils->runcmd($dcp_cmd, -1);
}

#-----------------------------------------------------------------------

=head3 check_disk_info

    check whether partition device name corresponds to its info
    on local Linux server

    Arguments:
        $cvals     reference to a hash that contains current MS definition

    Return:
        0       share disk meets its original info
        1       match failed

        $::INFO the info of possible partitions to be searched

=cut

#------------------------------------------------------------------------
sub check_disk_info
{
    my ($class, $cvals) = @_;
    my $file = "/var/opt/hams/sharedDiskInfo";

    # if the file storing info does not contain enough message,
    # we should skip the info examination
    if (!-f $file)
    {
        MessageUtils->message('W', 'EMsgInvalidInfoFile');
        return 0;
    }
    open(FILE, "<$file")
      or MessageUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
    my ($ms_name, $disk_info, $disk_name);
    while (my $line = <FILE>)
    {
        chomp $line;
        ($ms_name, $disk_info, $disk_name) = split('#', $line);
        if ($ms_name eq $$cvals{'hostname'}) { last; }
    }
    close(FILE);
    if ($disk_info !~ /\S+/g)
    {
        MessageUtils->message('W', 'EMsgInvalidInfoFile');
        return 0;
    }

    my $disk        = $$cvals{'PartitionDeviceName'};
    my $msdisk_info = "$ms_name#$disk_info";
    my $target_info = HAMS->get_info($disk, $cvals);
    if ($msdisk_info eq $target_info)
    {
        return 0;
    }

    $::INFO = $disk_info;
    return 1;
}

#----------------------------------------------------------------

=head3 store_disk_info

    store host, chan, id, lun and the sequence number of shared disk
    on both management servers on Linux

    Arguments:
        $cvals     reference to a hash that contains current MS info
        $rvals     reference to a hash that contains remote MS info

    Return:
        0          succeed
        1          fail
=cut

#----------------------------------------------------------------
sub store_disk_info
{
    my ($class, $cvals, $rvals) = @_;

    my $disk = $$cvals{'PartitionDeviceName'};
    my $info = HAMS->get_info($disk, $cvals, $rvals);
    if ($info eq 1)
    {
        return 1;
    }

    # create local definition file and copy it to remote MS
    my $file = "/var/opt/hams/sharedDiskInfo";
    open(HANDLE, ">$file")
      or MessageUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
    print HANDLE $info;
    close HANDLE;
    my $dcp_cmd .= "$::REMOTECOPYCMD $file $$rvals{'hostname'}\:$file";
    NodeUtils->runcmd($dcp_cmd, 0);
    return 0;
}

#----------------------------------------------------------------

=head3 start_FNMd

    start the FNM daemon on AIX only

    Arguments: none

    Return:
        0          succeeded
        1          failed
=cut

#----------------------------------------------------------------
sub start_FNMd
{
    my $cmd = "/opt/csm/hpsnm/bin/chswnm";
    if (NodeUtils->isAIX() && -e $cmd)
    {
        NodeUtils->runcmd("$cmd -b", -1);
        if ($::RUNCMD_RC != 0)
        {
            return 1;
        }
    }
    else
    {
        return 0;
    }
    return 0;
}

#----------------------------------------------------------------

=head3 stop_FNMd

    stop the FNM daemon on AIX only

    Arguments: none

    Return:
        0          succeeded
        1          failed
=cut

#----------------------------------------------------------------
sub stop_FNMd
{
    my $cmd = "/opt/csm/hpsnm/bin/chswnm";
    if (NodeUtils->isAIX() && -e $cmd)
    {
        my $output = NodeUtils->runcmd("$cmd -e", -1);

        # maybe FNM has been stopped already
        if ($::RUNCMD_RC != 0 && $output !~ m/already/)
        {
            # there is sth wrong stopping FNM
            # kill its processes
            HAMS->kill_service("fnmd");

            # recheck current FNMd status
            my @output = NodeUtils->runcmd("ps -ef|grep -E \"fnmd\|startFNMD\"", 0);
            if (scalar(@output) > 1 ||
                (scalar(@output) == 1 && $output[0] !~ m/grep -E fnmd\|startFNMD/))
            {
                MessageUtils->message('E', 'EMsgStoppingServiceFailed', 'HPSNM');
                return 1;
            }
        }
    }

    return 0;
}

#----------------------------------------------------------------

=head3 stop_HW_services

    stops the hardware related daemons

=cut

#----------------------------------------------------------------
sub stop_HW_services
{
    MessageUtils->message('I', 'IMsgStoppingHWServices');
    if ($::PLTFRM =~ m/AIX/)
    {
        HAMS->stop_FNMd();
    }
    if (   ($::PLTFRM =~ m/AIX/)
        || (($::PLTFRM =~ m/Linux/) && ($::HWARCH =~ m/ppc/)))
    {
        HAMS->stop_service("hdwr_svr");
        HAMS->stop_service("bpa_logger");
        HAMS->stop_service("tracelogd");
    }
    HAMS->stop_service("hwsdagent", 0);
    $::HW_STOPPED = 1;    #cache state
}

#----------------------------------------------------------------

=head3 start_HW_services

    starts the hardware related daemons

=cut

#----------------------------------------------------------------
sub start_HW_services
{
    MessageUtils->message('I', 'IMsgStartingHWServices');
    HAMS->start_service("hwsdagent", 0);
    if (   ($::PLTFRM =~ m/AIX/)
        || (($::PLTFRM =~ m/Linux/) && ($::HWARCH =~ m/ppc/)))
    {
        HAMS->start_service("tracelogd");
        HAMS->start_service("bpa_logger");
        HAMS->start_HDWR_SVR;

        if ($::PLTFRM =~ m/AIX/)
        {
            HAMS->start_FNMd();
        }
    }
    $::HW_STOPPED = 0;    #cache state
}

#----------------------------------------------------------------
sub start_HDWR_SVR
{
    if ($::PLTFRM =~ m/AIX/){
	  # check to see if service is defined
	  # if defined,  start it
	  # if not defined and was running on the primary MS
	  #    run configure hdr_svr
     if (HAMS->service_defined("hdwr_svr")) {
        HAMS->start_service("hdwr_svr");
	 } else { # not defined need to setup and start
		 my $cmd = "/opt/csm/csmbin/hdwr_svr_cfg";
		 if (!-x $cmd) {  # if command does not exist
                MessageUtils->message('E', 'EMsgCommandNotInstalled', $cmd);
		 } else {

	        my @output = NodeUtils->runcmd($cmd, -1);
			if ($::RUNCMD_RC != 0) {
              MessageUtils->message('E', 'EMsgCommandError', $cmd,$::RUNCMD_RC);
			}
		 }
         	
	 }


	} else {  # linux 

        HAMS->start_service("hdwr_svr");

     }
}


#----------------------------------------------------------------

=head3 stop_service
    stops the service with stopsrc (if the service is defined)
    Arguments: service name, runcmd error handling
=cut

#----------------------------------------------------------------
sub stop_service
{
    my ($class, $service, $rc) = @_;

    if (!defined $rc) { $rc = -1; }
    my $isStopped = 0;
    if (HAMS->service_defined($service))
    {
        my @output = NodeUtils->runcmd("LANG=C /usr/bin/lssrc -s $service", -1);
        if ($::RUNCMD_RC)
        {
            $rc = 0;
        }
        else
        {
            my ($subsys, $group, $pid, $status) = split(' ', $output[1]);
            my @strs = ($status, $pid, $group);
            foreach my $str (@strs)
            {
                if ($str =~ m/active/)
                {
                    $isStopped = 0;
                    last;
                }
                elsif ($str =~ m/inoperative/)
                {
                    $isStopped = 1;
                    last;
                }
                elsif ($str =~ m/\S+/)
                {
                    $rc = 0;
                    last;
                }
                else
                {
                    next;
                }
            }
        }

        if (!$isStopped)
        {
            eval {
                alarm(30);
                NodeUtils->runcmd("/usr/bin/stopsrc -s $service", $rc);
                alarm(0);
            };
            alarm(0);

            # ensure the service is stopped.
            HAMS->kill_service($service);
            HAMS->kill_service($service); # do again to kill spawned process
        }
    }
}

#----------------------------------------------------------------

=head3 start_service
    starts the service with startsrc (if the service is defined)
    Arguments: service name, runcmd error handling
=cut

#----------------------------------------------------------------
sub start_service
{
    my ($class, $service, $rc) = @_;
    if (!defined $rc) { $rc = -1; }
    if (HAMS->service_defined($service))
    {
        NodeUtils->runcmd("/usr/bin/startsrc -s $service", $rc);
    }
}

#----------------------------------------------------------------

=head3 service_defined
    indicates whether or not a service is defined in SRC
    
    returns: 0 = not defined
             1= defined
=cut

#----------------------------------------------------------------

sub service_defined
{
    my ($class, $service) = @_;
    if (!defined %::src_services)
    {
        %::src_services = ();
        my @output = NodeUtils->runcmd("/usr/bin/lssrc -a");
        foreach my $line (@output)
        {
            my @values = split /\s+/, $line;
            my $name = $values[1];
            $::src_services{$name} = 1;
        }
    }
    if (!defined $::src_services{$service})
    {
        return 0;
    }
    return 1;
}

#--------------------------------------------------------------------------------

=head3  touchFile

      Arguments:
        $filename
    Returns:
        non zero return code indicates error
    Globals:
        none
    Error:
        non zero return code
    Example:
        HAMS->touchFile("/var/opt/csm/touch");
=cut

#--------------------------------------------------------------------------------
sub touchFile
{
    my ($class, $filename) = @_;

    my $fh;
    my $rc = 0;
    open($fh, ">>$filename") or $rc++;
    close($fh) or $rc++;
    return $rc;

}

########################################################################

=head3

    isMountedonActive  check whether shared disk is mounted on Active MS

        arguments:
            $active    host name of active MS
            $cvals     reference to one hash for current MS
            $rvals     reference to one hash for remote MS

        returns:
            1 if mounted
            0 if not mounted

=cut

#########################################################################
sub isMountedonActive
{
    my ($class, $active, $cvals, $rvals) = @_;
    if ($active !~ m/\S+/)
    {
        $active = HAMS->determineActive($cvals, $rvals);
    }
    if ((!defined $active) || ($active eq "1"))
    {

        # neither MS is active
        return 0;
    }
    elsif ($active eq $$cvals{'hostname'})
    {

        # local server is active
        my $result = HAMS->isMountedonCurrent($cvals, $::MOUNT_POINT);
        return $result;
    }
    else
    {

        # remote server is active
        my $result = HAMS->isMountedonRemote($$rvals{'hostname'});
        return $result;
    }
}

########################################################################

=head3

    isMountedonRemote  check whether shared disk is mounted on Remote MS

    arguments:
      $remote    remote MS name

    returns:
      1 if mounted
      0 if not mounted

=cut

#########################################################################
sub isMountedonRemote
{
    my ($class, $remote) = @_;
    my $ismounted = 0;
    my @output    =
      NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $remote \"/bin/df -P $::MOUNT_POINT\"",
                        -1);
    if (scalar @output > 1)
    {
        my $found = 0;
        foreach my $line (@output)
        {
            HAMS->printDebug("DF line =  \"$line\"\n");
            my ($server, $file_sys, $blocks, $used, $avail, $cap, $mount_point)
              = split(' ', $line);
            if ($mount_point =~ m/$::MOUNT_POINT/)
            {
                HAMS->printDebug("line \"$line\" matches: $::MOUNT_POINT\n");
                $found = 1;
                my ($part, $rest) = split(/\s+/, $line, 2);
                last;
            }
        }
        if ($found)
        {
            $ismounted = 1;
        }
    }
    return $ismounted;
}

#########################################################################

=head3
    handleDiff  does prompting after a diff
    either prompts or exits with an error if the diff failed
=cut

#########################################################################
sub handleDiff
{

    if ($::RUNCMD_RC == 0) { return; }    #diff succeeded
    if (open(TTY, "/dev/tty"))
    {                                     #there is a terminal, prompt the user
        close TTY;
        MessageUtils->message('E', 'EMsgDiffFailed');
        MessageUtils->message('I', 'IMsgContinue');
        my $approved = 0;
        while (!$approved)
        {
            chomp(my $line = <STDIN>);
            if ($line =~ /n|N/)
            {
                exit 0;
            }
            elsif ($line =~ /y|Y/)
            {
                $approved = 1;
            }
            else
            {
                MessageUtils->message('I', 'IMsgContinue');
            }
        }
    }
    else
    {
        exit 2;    #diff failed and there is no terminal to prompt
    }
}

##################################################################

=head3
 isAlive   test whether one server/node is alive
           Both host name and IP address can be accepted.

 arguments:
    $host  target machine name or IP address

 return:
    1      alive
    0      unreachable

 For example:
    my $result = HAMS->isAlive($remote);

=cut

##################################################################
sub isAlive
{
    my ($class, $host) = @_;
    my $cmd = "ping -c 1 $host";
    NodeUtils->runcmd($cmd, -1);
    if ($::RUNCMD_RC)
    {
        return 0;
    }
    else
    {
        return 1;
    }
}

# return local directory space in bytes
# if links are passed into this routine,
# only their spaces of links, not the directories are returned
sub getDirSpace
{
    my ($class, @dirs) = @_;

    if (!scalar(@dirs))
    {
        return ("-1");
    }

    my (@output, @dir_spaces, $dir_str);
    $dir_str = join ' ', @dirs;
    if (NodeUtils->isLinux)
    {
        @output = NodeUtils->runcmd("/usr/bin/du -b -s $dir_str", 0);
        if ($::RUNCMD_RC)
        {
            return ("-1");
        }
        foreach my $line (@output)
        {
            my ($space, $directory) = split ' ', $line;
            push @dir_spaces, $space;
        }
    }
    elsif (NodeUtils->isAIX)
    {

        # du command on AIX does not accept -b option
        @output = NodeUtils->runcmd("/usr/bin/du -k -s $dir_str", 0);
        if ($::RUNCMD_RC)
        {
            return ("-1");
        }
        foreach my $line (@output)
        {
            my ($space, $directory) = split ' ', $line;
            $space *= 1024;    # transfer it into bytes
            push @dir_spaces, $space;
        }
    }
    else
    {
        return ("-1");
    }

    return @dir_spaces;
}

##############################################################
#
# get the unused space that contains the directory
#
# All the directories should meet 3 conditions
#
# 1. The directories should not contain special characters,
#    such as ' ', '.', and '_'.
# 2. If the directories are not '/',
#    they should not contain '/' at their ends.
# 3. The directories should lie in local disks.
#
###############################################################
sub getUnusedSpaceByDir
{
    my ($class, @dirs) = @_;

    my @output = NodeUtils->runcmd("/bin/df -k", 0);
    if ($::RUNCMD_RC)
    {
        return ("-1");
    }

    my (@possible_mount_points, @left_spaces, @file_systems);
    foreach my $line (@output)
    {
        HAMS->printDebug("DF line =  \"$line\"\n");
        my ($file_sys, $blocks, $free, $used, $Iused, $usedP, $IusedP,
            $mount_point);
        if (NodeUtils->isLinux())
        {
            ($file_sys, $blocks, $used, $free, $usedP, $mount_point) =
              split(' ', $line);
        }
        elsif (NodeUtils->isAIX())
        {
            ($file_sys, $blocks, $free, $used, $Iused, $IusedP, $mount_point) =
              split(' ', $line);
        }
        else
        {
            return ("-1");
        }

        $mount_point =~ s/\s+//;
        if ($mount_point !~ m/\S+/)
        {
            next;
        }

        my $i = 0;
        foreach my $dir (@dirs)
        {
            if ($dir =~ m/^$mount_point/ && $file_sys =~ m/^\/dev/)
            {
                HAMS->printDebug("Possible mount point $mount_point.\n");

                # choose the longest mount point for each directory
                my $local_dir = $dir;
                $local_dir =~ s/\s+//;
                my $j = 0;
                while (1)
                {
                    if ($local_dir eq $mount_point)
                    {
                        last;
                    }
                    elsif ($local_dir !~ m/[0-9 a-z A-Z]+/)

                      # it is / that contains this directory
                    {
                        $local_dir = "/";
                        last;
                    }
                    else

                      # remove the final section
                      # for example, /var/log/csm/hams --> /var/log/csm
                    {
                        $local_dir =~ s/\/[0-9 a-z A-Z]+$//;
                    }

                    # avoid never-ending recycle
                    $j++;
                    if ($j > 10000)
                    {
                        last;
                    }
                }

                if ($local_dir =~ m/\S+/
                    && length($local_dir) > length($possible_mount_points[$i]))
                {
                    $possible_mount_points[$i] = $local_dir;
                    $left_spaces[$i]           = $free * 1024;
                    $file_systems[$i]          = $file_sys;
                }
            }
            $i++;
        }
    }

    return (\@left_spaces, \@file_systems);
}

# only positive number is permitted
sub getSpaceMsg
{
    my ($class, $space) = @_;

    if ($space > 1073741824)
    {
        $space /= 1073741824;
        $space =~ s/\.\d+$//g;
        $space++;
        $space .= " G bytes";
    }
    elsif ($space > 1048576)
    {
        $space /= 1048576;
        $space =~ s/\.\d+$//g;
        $space++;
        $space .= " megabytes";
    }
    elsif ($space > 1024)
    {
        $space /= 1024;
        $space =~ s/\.\d+$//g;
        $space++;
        $space .= " kilobytes";
    }
    else
    {
        $space .= " bytes";
    }

    return $space;
}

# find out the process that belong to the service
# and kill them
sub kill_service
{
    my ($class, $service) = @_;

    my @output = NodeUtils->runcmd("ps -ef |grep $service", 0);
    my @pids;
    foreach my $line (@output)
    {
        if ($line =~ m/grep $service/)
        {
            next;
        }
        else
        {
            my ($user, $pid, $ppid, $num, $time, $con, $ctime, $cmd) =
              split ' ', $line;
            push @pids, $pid;
        }
    }

    my $service_pids = join ' ', @pids;
    my $return = 0;
    if ($service_pids =~ m/\d+/)
    {
        eval {
            alarm(60);
            NodeUtils->runcmd("/bin/kill -9 $service_pids", -1);
            if ($::RUNCMD_RC)
            {
                $return = 1;
            }
            alarm(0);
        };
        alarm(0);
    }

    return $return;
}

# get total space, NOT left space of one partition
# on AIX, this routine will umount the partition
sub getDiskSpace
{
    my ($class, $shared_partition, $cvals, $rvals) = @_;

    my ($disk_space);
    if (NodeUtils->isAIX())
    {
        if ($shared_partition eq $$cvals{'PartitionDeviceName'})
        {

            # BEGIN: varyon the VG so that we can access the LV
            my $current = $$cvals{'hostname'};
            my $remote  = $$rvals{'hostname'};

            #use alarm because sometimes dsh cannot kill child proc
            $SIG{ALRM} = sub { die "timeout expired\n" };
            eval {
                alarm(15);
                NodeUtils->runcmd(
                    "$::DSH -t $HAMS::TIMEOUT -vn $current,$remote \"/usr/sbin/umount /var/opt/hams/mnt\"",
                    -1
                    );
                alarm(0);
            };
            alarm(0);
            eval {
                alarm(15);
                NodeUtils->runcmd("/usr/sbin/varyoffvg $$cvals{'VolumeGroup'}",
                                  -1);
                alarm(0);
            };
            alarm(0);

            eval {
                alarm(15);
                NodeUtils->runcmd(
                    "$::REMOTE_SHELL $remote \"/usr/sbin/varyoffvg $$rvals{'VolumeGroup'}\"",
                    -1
                    );
                alarm(0);
            };
            alarm(0);
            NodeUtils->runcmd("/usr/sbin/varyonvg $$cvals{'VolumeGroup'}", -1);
            if ($::RUNCMD_RC != 0)
            {
                my @hdisk = HAMS->get_hdisk($$cvals{'VolumeGroup'});
                HAMS->break_reservation(@hdisk);
                NodeUtils->runcmd("/usr/sbin/varyonvg $$cvals{'VolumeGroup'}");
            }

            # END: varyon the VG so that we can access the LV
        }

        # find out disk space
        my $lv = $shared_partition;
        $lv =~ s/^\/dev\///;
        my @output = NodeUtils->runcmd("/usr/sbin/lslv -L $lv", 0);
        if ($::RUNCMD_RC)
        {
            return;
        }
        @output = grep /LPs/, @output;

        # @output should be in this style
        # MAX LPs:            1024                   PP SIZE:        32 megabyte(s)
        # LPs:                501                    PPs:            501
        my ($pp_size, $lp_num);
        foreach my $line (@output)
        {
            if ($line =~ m/PP SIZE:/)
            {
                my ($words,   $size) = split 'PP SIZE:', $line;
                my ($PP_size, $unit) = split ' ',        $size;
                if ($PP_size =~ m/\S+/ && $unit =~ m/\S+/)
                {
                    if ($unit =~ m/^k/)
                    {
                        $PP_size *= 1024;
                    }
                    elsif ($unit =~ m/^m/)
                    {
                        $PP_size *= 1024 * 1024;
                    }
                    elsif ($unit =~ m/^g/)
                    {
                        $PP_size *= 1024 * 1024 * 1024;
                    }
                }
                $pp_size = $PP_size;
            }
            elsif ($line =~ m/PPs:/)
            {
                my ($LPs, $LP_num, $PPs, $PP_num) = split ' ', $line;
                $lp_num = $LP_num;
            }
        }
        if ($pp_size =~ m/\S+/ && $lp_num =~ m/\S+/)
        {
            $disk_space = $pp_size * $lp_num;
        }
    }
    elsif (NodeUtils->isLinux())
    {

        # Linux section begin
        # find out disk space
        my $disk = $shared_partition;
        $disk =~ s/\d+$//;
        my @output = NodeUtils->runcmd("/sbin/sfdisk -l $disk", 0);
        if ($::RUNCMD_RC)
        {
            return;
        }
        my ($block_size, $block_num);
        foreach my $line (@output)
        {
            if ($line =~ m/Units =/)
            {

                # one example:  Units = cylinders of 16065 * 512 = 8225280 bytes
                my ($pre_words, $size, $post_words) = split('bytes', $line);
                $size =~ s/\D+//g;
                $block_size = $size;
            }
            elsif ($line =~ m/$shared_partition/)
            {
                my ($device, $start, $end, $syl, $num, $id, $system) =
                  split ' ', $line;

                # example line:
                # /dev/sda1          0+   1008    1009-   2095662   83  Linux
                if ($device ne $shared_partition)
                {
                    next;
                }
                if ($start eq "*")

                  # if the disk is bootable, there will be one more row than normal.
                  # example line:
                  # /dev/sda1   *      0+    332     333-    691610   83  Linux
                {
                    $num = $id;
                }
                $num =~ s/\s+//g;
                $num =~ s/\+//g;
                $block_num = $num;
            }
        }
        if ($block_size =~ m/\S+/ && $block_num =~ m/\S+/)
        {
            $disk_space = $block_size * $block_num;
        }
    }    # Linux section end

    return $disk_space;
}

################################################################
#
#   isCSMInstallPartitionMounted
#       check whether local CSMInstallPartition is mounted on /csminstall
#
#   arguments:
#       $partition     the contents of $$cvals{'CSMInstallPartition')
#
#   returns:
#       1 if mounted
#       0 if not mounted
#
################################################################
sub isCSMInstallPartitionMounted
{
    my ($class, $partition) = @_;

    HAMS->printDebug("inside HAMS->isMountedonCurrent\n");
    my $dir   = "/csminstall";
    my @output = NodeUtils->runcmd("/bin/df -P $dir");
    if (scalar @output < 1)
    {
        return 0;
    }

    my $found = 0;
    foreach my $line (@output)
    {
        my ($file_sys, $blocks, $used, $avail, $cap, $mount_point) =
          split(' ', $line);
        if ($file_sys eq $partition && $mount_point eq $dir)
        {
            $found = 1;
            last;
        }
    }

    return $found;
}
#######################################################################
# kill_processes 
# This routine kills processes accessing directories
#
# Input the directory that we want all the processes removed
#######################################################################
sub kill_processes 
{
    my ($class, $directory) = @_; 
	my $cmd = "LANG=C /usr/sbin/fuser -uxc $directory";
	my @output = NodeUtils->runcmd($cmd, 0);
	if ($::RUNCMD_RC eq 0)
	{
	   foreach my $line (@output)
	   {
		   if ($line =~ m/\S+\s+\d+/)

		   {
			 # this line is in such format
			 # /csminstall:    13850(root)   28310(root
			 my ($dir, $pids) = split(':', $line);
			 $pids =~ s/\D+/\ /g;
			 if ($pids !~ m/\d+/)
			 {
				 return 0;
			 }
                         $cmd="/bin/kill -9 $pids";
                         NodeUtils->runcmd($cmd, 0);  
                         MessageUtils->message('LI', 'IMsgKillPID',$pids);
		   }
	   }
	}
    return 0;
}

########################################################################
#
# recordDhcpStatus
#
#   recored current dhcp status in one global variable,
#   so that it can be restarted later
#
# Global:
#   $HAMS::DHCP_RUNNING    1 running
#                          0 not running
#
########################################################################
sub recordDhcpStatus
{
    if ($HAMS::DHCP_RUNNING)
    # use cached status
    {
        return 0;
    }

    if (NodeUtils->isAIX() && HAMS->service_defined("dhcpsd"))
    # on AIX
    {
        my @output = NodeUtils->runcmd("LANG=C /usr/bin/lssrc -s dhcpsd ",0);
        if ($::RUNCMD_RC != 0)
        # error setting up dhcpsd
        # could not determine whether we should start
        {
            MessageUtils->message('I', 'IMsgdhcpsd');
        }
        else
        {
            # see if dhcp is operative
            my ($subsys, $group, $pid, $status) = split(' ', $output[1]);
            if (defined($status) && $status eq 'active')
            {
                $HAMS::DHCP_RUNNING = 1;
            }
        }
    }

    # on Linux, nothing is required because dhcp service
    # will not be interrupted by HA MS
}

########################################################################
#
# restoreDhcp
#   if dhcp was running, restart it.
#   otherwise, do nothing
#
# Global:
#   $HAMS::DHCP_RUNNING     1 running
#                           0 not running
#
########################################################################
sub restoreDhcp
{
    if (!$HAMS::DHCP_RUNNING)
    {
        return 0;
    }

    if (NodeUtils->isAIX())
    {
        HAMS->start_service("dhcpsd", -1);
    # if dhcpsd has been running, ignore the warning
    }

    # on Linux, nothing is required because dhcp service
    # will not be interrupted by HA MS
}

1;
